
from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import (
    SharedWebServer,
    SharedWebServerBulkRequest,
    SharedWebServerBulkResponse,
)


class SharedWebServerService(BaseService):

    @cast_models
    def get_shared_web_server(self, id_: str) -> Union[SharedWebServer, str]:
        """Retrieves the details of a Shared Web Server configuration for this atom/cloud ID by its unique ID. The response can be in either XML or JSON format based on your request.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedWebServer, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedWebServer/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedWebServer._unmap(response)
        if content == "application/xml":
            return SharedWebServer._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_shared_web_server(
        self, id_: str, request_body: SharedWebServer = None
    ) -> Union[SharedWebServer, str]:
        """Updates a Shared Web Server object based on the supplied Runtime ID.

        :param request_body: The request body., defaults to None
        :type request_body: SharedWebServer, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedWebServer, str]
        """

        Validator(SharedWebServer).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedWebServer/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedWebServer._unmap(response)
        if content == "application/xml":
            return SharedWebServer._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_shared_web_server(
        self, request_body: SharedWebServerBulkRequest = None
    ) -> Union[SharedWebServerBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: SharedWebServerBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedWebServerBulkResponse, str]
        """

        Validator(SharedWebServerBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedWebServer/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedWebServerBulkResponse._unmap(response)
        if content == "application/xml":
            return SharedWebServerBulkResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment_map_extension_external_component import (
    EnvironmentMapExtensionExternalComponentService,
)
from ...models import (
    EnvironmentMapExtensionExternalComponentQueryResponse,
    EnvironmentMapExtensionExternalComponentQueryConfig,
)


class EnvironmentMapExtensionExternalComponentServiceAsync(
    EnvironmentMapExtensionExternalComponentService
):
    """
    Async Wrapper for EnvironmentMapExtensionExternalComponentServiceAsync
    """

    def query_environment_map_extension_external_component(
        self, request_body: EnvironmentMapExtensionExternalComponentQueryConfig = None
    ) -> Awaitable[Union[EnvironmentMapExtensionExternalComponentQueryResponse, str]]:
        return to_async(super().query_environment_map_extension_external_component)(
            request_body
        )

    def query_more_environment_map_extension_external_component(
        self, request_body: str
    ) -> Awaitable[Union[EnvironmentMapExtensionExternalComponentQueryResponse, str]]:
        return to_async(
            super().query_more_environment_map_extension_external_component
        )(request_body)




from typing import Awaitable
from .utils.to_async import to_async
from ..account_provision import AccountProvisionService


class AccountProvisionServiceAsync(AccountProvisionService):
    """
    Async Wrapper for AccountProvisionServiceAsync
    """

    def create_account_provision(self) -> Awaitable[None]:
        return to_async(super().create_account_provision)()




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .field import Field


@JsonMap({"id_": "id", "type_": "type"})
class SharedCommunication(BaseModel):
    """SharedCommunication

    :param field: field, defaults to None
    :type field: List[Field], optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param type_: type_, defaults to None
    :type type_: str, optional
    """

    def __init__(
        self,
        field: List[Field] = SENTINEL,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        type_: str = SENTINEL,
        **kwargs,
    ):
        """SharedCommunication

        :param field: field, defaults to None
        :type field: List[Field], optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param name: name, defaults to None
        :type name: str, optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        """
        if field is not SENTINEL:
            self.field = self._define_list(field, Field)
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        if type_ is not SENTINEL:
            self.type_ = type_
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .account_user_federation_expression import (
    AccountUserFederationExpression,
    AccountUserFederationExpressionGuard,
)
from .account_user_federation_simple_expression import (
    AccountUserFederationSimpleExpression,
)
from .account_user_federation_grouping_expression import (
    AccountUserFederationGroupingExpression,
)


@JsonMap({})
class AccountUserFederationQueryConfigQueryFilter(BaseModel):
    """AccountUserFederationQueryConfigQueryFilter

    :param expression: expression
    :type expression: AccountUserFederationExpression
    """

    def __init__(self, expression: AccountUserFederationExpression, **kwargs):
        """AccountUserFederationQueryConfigQueryFilter

        :param expression: expression
        :type expression: AccountUserFederationExpression
        """
        self.expression = AccountUserFederationExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AccountUserFederationQueryConfig(BaseModel):
    """AccountUserFederationQueryConfig

    :param query_filter: query_filter
    :type query_filter: AccountUserFederationQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: AccountUserFederationQueryConfigQueryFilter, **kwargs
    ):
        """AccountUserFederationQueryConfig

        :param query_filter: query_filter
        :type query_filter: AccountUserFederationQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, AccountUserFederationQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from typing import Awaitable
from .utils.to_async import to_async
from ..shared_web_server_log import SharedWebServerLogService


class SharedWebServerLogServiceAsync(SharedWebServerLogService):
    """
    Async Wrapper for SharedWebServerLogServiceAsync
    """

    def create_shared_web_server_log(self) -> Awaitable[None]:
        return to_async(super().create_shared_web_server_log)()




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "account_id": "accountId",
        "first_name": "firstName",
        "id_": "id",
        "last_name": "lastName",
        "notify_user": "notifyUser",
        "role_id": "roleId",
        "user_id": "userId",
    }
)
class AccountUserRole(BaseModel):
    """AccountUserRole

    :param account_id: The account ID., defaults to None
    :type account_id: str, optional
    :param first_name: The first name of a user in the account., defaults to None
    :type first_name: str, optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param last_name: The last name of a user in the account., defaults to None
    :type last_name: str, optional
    :param notify_user: If true, defaults to None
    :type notify_user: bool, optional
    :param role_id: The ID of the role., defaults to None
    :type role_id: str, optional
    :param user_id: The user ID., defaults to None
    :type user_id: str, optional
    """

    def __init__(
        self,
        account_id: str = SENTINEL,
        first_name: str = SENTINEL,
        id_: str = SENTINEL,
        last_name: str = SENTINEL,
        notify_user: bool = SENTINEL,
        role_id: str = SENTINEL,
        user_id: str = SENTINEL,
        **kwargs
    ):
        """AccountUserRole

        :param account_id: The account ID., defaults to None
        :type account_id: str, optional
        :param first_name: The first name of a user in the account., defaults to None
        :type first_name: str, optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param last_name: The last name of a user in the account., defaults to None
        :type last_name: str, optional
        :param notify_user: If true, defaults to None
        :type notify_user: bool, optional
        :param role_id: The ID of the role., defaults to None
        :type role_id: str, optional
        :param user_id: The user ID., defaults to None
        :type user_id: str, optional
        """
        if account_id is not SENTINEL:
            self.account_id = account_id
        if first_name is not SENTINEL:
            self.first_name = first_name
        if id_ is not SENTINEL:
            self.id_ = id_
        if last_name is not SENTINEL:
            self.last_name = last_name
        if notify_user is not SENTINEL:
            self.notify_user = notify_user
        if role_id is not SENTINEL:
            self.role_id = role_id
        if user_id is not SENTINEL:
            self.user_id = user_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .as2_partner_info import As2PartnerInfo
from .as2_mdn_options import As2MdnOptions
from .as2_message_options import As2MessageOptions
from .as2_my_company_info import As2MyCompanyInfo


@JsonMap(
    {
        "as2_default_partner_info": "AS2DefaultPartnerInfo",
        "as2_default_partner_mdn_options": "AS2DefaultPartnerMDNOptions",
        "as2_default_partner_message_options": "AS2DefaultPartnerMessageOptions",
        "as2_my_company_info": "AS2MyCompanyInfo",
    }
)
class As2ReceiveOptions(BaseModel):
    """As2ReceiveOptions

    :param as2_default_partner_info: as2_default_partner_info
    :type as2_default_partner_info: As2PartnerInfo
    :param as2_default_partner_mdn_options: as2_default_partner_mdn_options
    :type as2_default_partner_mdn_options: As2MdnOptions
    :param as2_default_partner_message_options: as2_default_partner_message_options
    :type as2_default_partner_message_options: As2MessageOptions
    :param as2_my_company_info: as2_my_company_info
    :type as2_my_company_info: As2MyCompanyInfo
    """

    def __init__(
        self,
        as2_default_partner_info: As2PartnerInfo,
        as2_default_partner_mdn_options: As2MdnOptions,
        as2_default_partner_message_options: As2MessageOptions,
        as2_my_company_info: As2MyCompanyInfo,
        **kwargs,
    ):
        """As2ReceiveOptions

        :param as2_default_partner_info: as2_default_partner_info
        :type as2_default_partner_info: As2PartnerInfo
        :param as2_default_partner_mdn_options: as2_default_partner_mdn_options
        :type as2_default_partner_mdn_options: As2MdnOptions
        :param as2_default_partner_message_options: as2_default_partner_message_options
        :type as2_default_partner_message_options: As2MessageOptions
        :param as2_my_company_info: as2_my_company_info
        :type as2_my_company_info: As2MyCompanyInfo
        """
        self.as2_default_partner_info = self._define_object(
            as2_default_partner_info, As2PartnerInfo
        )
        self.as2_default_partner_mdn_options = self._define_object(
            as2_default_partner_mdn_options, As2MdnOptions
        )
        self.as2_default_partner_message_options = self._define_object(
            as2_default_partner_message_options, As2MessageOptions
        )
        self.as2_my_company_info = self._define_object(
            as2_my_company_info, As2MyCompanyInfo
        )
        self._kwargs = kwargs




from ...models.utils.base_model import BaseModel
from .response import Response
from typing import Optional


class ApiError(BaseModel, Exception):
    """
    Class representing an API Error.

    :ivar Optional[int] status: The status code of the HTTP error.
    :ivar Optional[Response] response: The response associated with the error.
    :ivar Optional[str] error_detail: Parsed error message from response body (if available).
    """

    def __init__(
        self,
        message: Optional[str] = None,
        status: Optional[int] = None,
        response: Optional[Response] = None,
    ):
        """
        Initialize a new instance of API Error.

        :param Optional[str] message: The error message.
        :param Optional[int] status: The status code of the HTTP error.
        :param Optional[Response] response: The response associated with the error.
        """
        self.message = message
        self.status = status
        self.response = response
        self.error_detail = self._extract_error_detail()

    def _extract_error_detail(self) -> Optional[str]:
        """
        Extract detailed error message from XML response body if available.

        Returns:
            The error message from <error><message>...</message></error> or None
        """
        if not self.response or not hasattr(self.response, 'body'):
            return None

        body = self.response.body
        if not body or not isinstance(body, str):
            return None

        # Only try to parse XML error responses
        if not body.strip().startswith('<?xml') and not body.strip().startswith('<error'):
            return None

        try:
            from .utils import parse_xml_to_dict
            parsed = parse_xml_to_dict(body)

            # Check for standard Boomi error format: <error><message>...</message></error>
            if isinstance(parsed, dict) and 'error' in parsed:
                error_data = parsed['error']
                if isinstance(error_data, dict) and 'message' in error_data:
                    return error_data['message']

        except Exception:
            # If parsing fails, just return None
            pass

        return None




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .component_metadata_simple_expression import ComponentMetadataSimpleExpression
from .component_metadata_grouping_expression import ComponentMetadataGroupingExpression


class ComponentMetadataExpressionGuard(OneOfBaseModel):
    class_list = {
        "ComponentMetadataSimpleExpression": ComponentMetadataSimpleExpression,
        "ComponentMetadataGroupingExpression": ComponentMetadataGroupingExpression,
    }


ComponentMetadataExpression = Union[
    ComponentMetadataSimpleExpression, ComponentMetadataGroupingExpression
]




from enum import Enum
from .sentinel import was_value_set


class JsonMap:
    """
    A class decorator used to map adjusted attribute names to original JSON attribute names before a request,
    and vice versa after the request.

    Example:
    @JsonMapping({
        'adjusted_name': 'original_name',
        'adjusted_list': 'original_list'
    })
    class SomeClass(BaseModel):
        adjusted_name: str
        adjusted_list: List[OtherClass]

    :param mapping: A dictionary specifying the mapping between adjusted attribute names and original JSON attribute names.
    :type mapping: dict
    """

    def __init__(self, mapping):
        self.mapping = mapping

    def __call__(self, cls):
        """
        Transform the decorated class with attribute mapping capabilities.

        :param cls: The class to be decorated.
        :type cls: type
        :return: The decorated class.
        :rtype: type
        """
        cls.__json_mapping = self.mapping

        def _map(self):
            """
            Convert the object's attributes to a dictionary with mapped attribute names.

            :return: A dictionary with mapped attribute names and values.
            :rtype: dict
            """
            map = self.__json_mapping
            attribute_dict = vars(self)
            result_dict = {}

            for key, value in attribute_dict.items():
                if key == "_kwargs" or not was_value_set(value):
                    continue
                if isinstance(value, list):
                    value = [v._map() if hasattr(v, "_map") else v for v in value]
                elif isinstance(value, Enum):
                    value = value.value
                elif hasattr(value, "_map"):
                    value = value._map()
                mapped_key = map.get(key, key)
                result_dict[mapped_key] = value

            return result_dict

        @classmethod
        def _unmap(cls, mapped_data):
            """
            Create an object instance from a dictionary with mapped attribute names.

            :param mapped_data: A dictionary with mapped attribute names and values.
            :type mapped_data: dict
            :return: An instance of the class with attribute values assigned from the dictionary.
            :rtype: cls
            """
            reversed_map = {v: k for k, v in cls.__json_mapping.items()}
            mapped_attributes = {}

            for key, value in mapped_data.items():
                mapped_key = reversed_map.get(key, key)
                mapped_attributes[mapped_key] = value

            return cls(**mapped_attributes)

        cls._map = _map
        cls._unmap = _unmap

        return cls




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .browse_field import BrowseField


@JsonMap({"browse_field": "BrowseField", "connection_id": "connectionId"})
class MapExtensionBrowseData(BaseModel):
    """Fields defining the credentials for connecting to the external service for the purpose of reimporting the source of destination profile to retrieve custom fields. You use these fields in the Environment Map Extension object's EXECUTE action.

    :param browse_field: browse_field, defaults to None
    :type browse_field: List[BrowseField], optional
    :param connection_id: connection_id, defaults to None
    :type connection_id: str, optional
    """

    def __init__(
        self,
        browse_field: List[BrowseField] = SENTINEL,
        connection_id: str = SENTINEL,
        **kwargs,
    ):
        """Fields defining the credentials for connecting to the external service for the purpose of reimporting the source of destination profile to retrieve custom fields. You use these fields in the Environment Map Extension object's EXECUTE action.

        :param browse_field: browse_field, defaults to None
        :type browse_field: List[BrowseField], optional
        :param connection_id: connection_id, defaults to None
        :type connection_id: str, optional
        """
        if browse_field is not SENTINEL:
            self.browse_field = self._define_list(browse_field, BrowseField)
        if connection_id is not SENTINEL:
            self.connection_id = connection_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .oftp_listen_options_info import OftpListenOptionsInfo
from .oftp_partner_group_type import OftpPartnerGroupType


@JsonMap(
    {
        "oftp_listen_options": "OFTPListenOptions",
        "oftp_partner_group": "OFTPPartnerGroup",
        "partner_group_id": "partnerGroupId",
    }
)
class OftpListenOptions(BaseModel):
    """OftpListenOptions

    :param oftp_listen_options: oftp_listen_options
    :type oftp_listen_options: OftpListenOptionsInfo
    :param oftp_partner_group: oftp_partner_group
    :type oftp_partner_group: OftpPartnerGroupType
    :param partner_group_id: partner_group_id, defaults to None
    :type partner_group_id: str, optional
    """

    def __init__(
        self,
        oftp_listen_options: OftpListenOptionsInfo,
        oftp_partner_group: OftpPartnerGroupType,
        partner_group_id: str = SENTINEL,
        **kwargs,
    ):
        """OftpListenOptions

        :param oftp_listen_options: oftp_listen_options
        :type oftp_listen_options: OftpListenOptionsInfo
        :param oftp_partner_group: oftp_partner_group
        :type oftp_partner_group: OftpPartnerGroupType
        :param partner_group_id: partner_group_id, defaults to None
        :type partner_group_id: str, optional
        """
        self.oftp_listen_options = self._define_object(
            oftp_listen_options, OftpListenOptionsInfo
        )
        self.oftp_partner_group = self._define_object(
            oftp_partner_group, OftpPartnerGroupType
        )
        if partner_group_id is not SENTINEL:
            self.partner_group_id = partner_group_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .audit_log_simple_expression import AuditLogSimpleExpression
from .audit_log_grouping_expression import AuditLogGroupingExpression


class AuditLogExpressionGuard(OneOfBaseModel):
    class_list = {
        "AuditLogSimpleExpression": AuditLogSimpleExpression,
        "AuditLogGroupingExpression": AuditLogGroupingExpression,
    }


AuditLogExpression = Union[AuditLogSimpleExpression, AuditLogGroupingExpression]




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"implied_decimal": "impliedDecimal"})
class MapExtensionsNumber(BaseModel):
    """MapExtensionsNumber

    :param format: format, defaults to None
    :type format: str, optional
    :param implied_decimal: implied_decimal, defaults to None
    :type implied_decimal: int, optional
    :param signed: signed, defaults to None
    :type signed: bool, optional
    """

    def __init__(
        self,
        format: str = SENTINEL,
        implied_decimal: int = SENTINEL,
        signed: bool = SENTINEL,
        **kwargs
    ):
        """MapExtensionsNumber

        :param format: format, defaults to None
        :type format: str, optional
        :param implied_decimal: implied_decimal, defaults to None
        :type implied_decimal: int, optional
        :param signed: signed, defaults to None
        :type signed: bool, optional
        """
        if format is not SENTINEL:
            self.format = format
        if implied_decimal is not SENTINEL:
            self.implied_decimal = implied_decimal
        if signed is not SENTINEL:
            self.signed = signed
        self._kwargs = kwargs




from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models


class ExecuteProcessService(BaseService):

    @cast_models
    def create_execute_process(self) -> None:
        """This API is documented externally. Please visit the following link for full details:

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ExecuteProcess",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
        )

        self.send_request(serialized_request)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..tradacoms_connector_record import TradacomsConnectorRecordService
from ...models import (
    TradacomsConnectorRecordQueryResponse,
    TradacomsConnectorRecordQueryConfig,
)


class TradacomsConnectorRecordServiceAsync(TradacomsConnectorRecordService):
    """
    Async Wrapper for TradacomsConnectorRecordServiceAsync
    """

    def query_tradacoms_connector_record(
        self, request_body: TradacomsConnectorRecordQueryConfig = None
    ) -> Awaitable[Union[TradacomsConnectorRecordQueryResponse, str]]:
        return to_async(super().query_tradacoms_connector_record)(request_body)

    def query_more_tradacoms_connector_record(
        self, request_body: str
    ) -> Awaitable[Union[TradacomsConnectorRecordQueryResponse, str]]:
        return to_async(super().query_more_tradacoms_connector_record)(request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .process_property import ProcessProperty


@JsonMap({"process_property": "ProcessProperty"})
class ExecutionRequestProcessProperties(BaseModel):
    """The full list of Process properties within the specified Runtime, Runtime cluster, or cloud, where each property is defined by their name and value.

    :param process_property: process_property, defaults to None
    :type process_property: List[ProcessProperty], optional
    """

    def __init__(self, process_property: List[ProcessProperty] = SENTINEL, **kwargs):
        """The full list of Process properties within the specified Runtime, Runtime cluster, or cloud, where each property is defined by their name and value.

        :param process_property: process_property, defaults to None
        :type process_property: List[ProcessProperty], optional
        """
        if process_property is not SENTINEL:
            self.process_property = self._define_list(process_property, ProcessProperty)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .hd_type import HdType
from .processing_type import ProcessingType


@JsonMap(
    {
        "application": "Application",
        "facility": "Facility",
        "network_address": "NetworkAddress",
        "processing_id": "ProcessingId",
    }
)
class MshControlInfo(BaseModel):
    """MshControlInfo

    :param application: application
    :type application: HdType
    :param facility: facility
    :type facility: HdType
    :param network_address: network_address
    :type network_address: HdType
    :param processing_id: processing_id
    :type processing_id: ProcessingType
    """

    def __init__(
        self,
        application: HdType,
        facility: HdType,
        network_address: HdType,
        processing_id: ProcessingType,
        **kwargs,
    ):
        """MshControlInfo

        :param application: application
        :type application: HdType
        :param facility: facility
        :type facility: HdType
        :param network_address: network_address
        :type network_address: HdType
        :param processing_id: processing_id
        :type processing_id: ProcessingType
        """
        self.application = self._define_object(application, HdType)
        self.facility = self._define_object(facility, HdType)
        self.network_address = self._define_object(network_address, HdType)
        self.processing_id = self._define_object(processing_id, ProcessingType)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..edifact_connector_record import EdifactConnectorRecordService
from ...models import (
    EdifactConnectorRecordQueryResponse,
    EdifactConnectorRecordQueryConfig,
)


class EdifactConnectorRecordServiceAsync(EdifactConnectorRecordService):
    """
    Async Wrapper for EdifactConnectorRecordServiceAsync
    """

    def query_edifact_connector_record(
        self, request_body: EdifactConnectorRecordQueryConfig = None
    ) -> Awaitable[Union[EdifactConnectorRecordQueryResponse, str]]:
        return to_async(super().query_edifact_connector_record)(request_body)

    def query_more_edifact_connector_record(
        self, request_body: str
    ) -> Awaitable[Union[EdifactConnectorRecordQueryResponse, str]]:
        return to_async(super().query_more_edifact_connector_record)(request_body)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..atom_security_policies import AtomSecurityPoliciesService
from ...models import (
    AtomSecurityPolicies,
    AsyncOperationTokenResult,
    AtomSecurityPoliciesAsyncResponse,
)


class AtomSecurityPoliciesServiceAsync(AtomSecurityPoliciesService):
    """
    Async Wrapper for AtomSecurityPoliciesServiceAsync
    """

    def update_atom_security_policies(
        self, id_: str, request_body: AtomSecurityPolicies = None
    ) -> Awaitable[Union[AtomSecurityPolicies, str]]:
        return to_async(super().update_atom_security_policies)(id_, request_body)

    def async_get_atom_security_policies(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_atom_security_policies)(id_)

    def async_token_atom_security_policies(
        self, token: str
    ) -> Awaitable[Union[AtomSecurityPoliciesAsyncResponse, str]]:
        return to_async(super().async_token_atom_security_policies)(token)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AsyncOperationTokenResult,
    Atom,
    AtomBulkRequest,
    AtomBulkResponse,
    AtomCountersAsyncResponse,
    AtomQueryConfig,
    AtomQueryResponse,
    PersistedProcessPropertiesAsyncResponse,
)


class AtomService(BaseService):

    @cast_models
    def create_atom(self, request_body: Atom = None) -> Union[Atom, str]:
        """Creates and attaches a Runtime with the specified name to a specified Runtime cloud owned by the requesting account. This operation cannot be used to create a local Runtime. You must have the Runtime Management privilege to perform the POST operation.

         >**Note:** The `createdBy` is a system-generated or read-only field. It cannot be passed in a CREATE request.

        :param request_body: The request body., defaults to None
        :type request_body: Atom, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Atom, str]
        """

        Validator(Atom).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Atom",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Atom._unmap(response)
        if content == "application/xml":
            return Atom._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_atom(self, id_: str) -> Union[Atom, str]:
        """Retrieves the properties of the Runtime, Runtime cluster, or Runtime cloud having the specified ID.

         For Runtime clusters and Runtime clouds that are part of a multi-node runtime, the GET operation returns values for the following additional variables:

          - *nodeId*
          - *hostName*
          - *status*
          - *clusterProblem*

          For more information on these variables, see the topic [Cluster Status panel](https://help.boomi.com/docs/Atomsphere/Integration/Integration%20management/r-atm-Cluster_Status_panel_fbdb3645-00e2-4c3c-bba8-bf5fdb0f90f6).

        :param id_: A unique ID for the Runtime, Runtime cluster, or Runtime cloud.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Atom, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Atom/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Atom._unmap(response)
        if content == "application/xml":
            return Atom._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_atom(self, id_: str, request_body: Atom = None) -> Union[Atom, str]:
        """Updates the Runtime object having the specified ID. You can only update the name, purgeHistoryDays, purgeImmediate, forceRestartTime. You must have the Runtime Management privilege to perform the UPDATE operation. If you have the Runtime Management Read Access privilege, you cannot update an Runtime.

         >**Note:** There might be a delay before you see the changes in the Runtime Information panel.

        :param request_body: The request body., defaults to None
        :type request_body: Atom, optional
        :param id_: A unique ID for the Runtime, Runtime cluster, or Runtime cloud.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Atom, str]
        """

        Validator(Atom).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Atom/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Atom._unmap(response)
        if content == "application/xml":
            return Atom._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_atom(self, id_: str) -> None:
        """Deletes the Runtime object with the specified ID. You must have the Runtime Management privilege to perform the DELETE operation. If you have the Runtime Management Read Access privilege, you cannot delete a Runtime.

        :param id_: A unique ID for the Runtime, Runtime cluster, or Runtime cloud.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Atom/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_atom(
        self, request_body: AtomBulkRequest = None
    ) -> Union[AtomBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: AtomBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomBulkResponse, str]
        """

        Validator(AtomBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Atom/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomBulkResponse._unmap(response)
        if content == "application/xml":
            return AtomBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_atom(
        self, request_body: AtomQueryConfig = None
    ) -> Union[AtomQueryResponse, str]:
        """Use either `BROKER` or `GATEWAY` with either the CONTAINS or NOT_CONTAINS operator to filter by API Gateways and Authentication Brokers that you own.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: AtomQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomQueryResponse, str]
        """

        Validator(AtomQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Atom/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomQueryResponse._unmap(response)
        if content == "application/xml":
            return AtomQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_atom(self, request_body: str) -> Union[AtomQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Atom/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomQueryResponse._unmap(response)
        if content == "application/xml":
            return AtomQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_token_atom_counters(
        self, token: str
    ) -> Union[AtomCountersAsyncResponse, str]:
        """For a response, use the token from the initial GET response in a new request.

        :param token: Takes in the token from a previous call to return a result.
        :type token: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomCountersAsyncResponse, str]
        """

        Validator(str).validate(token)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/AtomCounters/response/{{token}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("token", token)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomCountersAsyncResponse._unmap(response)
        if content == "application/xml":
            return AtomCountersAsyncResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_get_atom_counters(
        self, id_: str
    ) -> Union[AsyncOperationTokenResult, str]:
        """The GET operation returns the current state of the counter names and values for the specified Runtime. The initial GET operation returns a token for the specified Runtime.
         `accountId` is the Platform account that you are authenticating with and `id` is the Runtime ID for the counters you are attempting to GET.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AsyncOperationTokenResult, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/AtomCounters/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AsyncOperationTokenResult._unmap(response)
        if content == "application/xml":
            return AsyncOperationTokenResult._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_token_persisted_process_properties(
        self, token: str
    ) -> Union[PersistedProcessPropertiesAsyncResponse, str]:
        """For a response, use the token from the response in a new request.

        :param token: Takes in the token from a previous call to return a result.
        :type token: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PersistedProcessPropertiesAsyncResponse, str]
        """

        Validator(str).validate(token)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/PersistedProcessProperties/response/{{token}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("token", token)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PersistedProcessPropertiesAsyncResponse._unmap(response)
        if content == "application/xml":
            return PersistedProcessPropertiesAsyncResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_get_persisted_process_properties(
        self, id_: str
    ) -> Union[AsyncOperationTokenResult, str]:
        """The GET operation returns the current state of the Persisted Process properties names and values for the specified Runtime.
         The initial GET operation returns a token for the specified Runtime.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AsyncOperationTokenResult, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/PersistedProcessProperties/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AsyncOperationTokenResult._unmap(response)
        if content == "application/xml":
            return AsyncOperationTokenResult._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"worker_id": "workerId"})
class AtomWorkerLog(BaseModel):
    """AtomWorkerLog

    :param worker_id: The name of an execution worker. Locate the name of an execution worker by navigating to the Runtime Workers panel in Manage \> Runtime Management on the user interface., defaults to None
    :type worker_id: str, optional
    """

    def __init__(self, worker_id: str = SENTINEL, **kwargs):
        """AtomWorkerLog

        :param worker_id: The name of an execution worker. Locate the name of an execution worker by navigating to the Runtime Workers panel in Manage \> Runtime Management on the user interface., defaults to None
        :type worker_id: str, optional
        """
        if worker_id is not SENTINEL:
            self.worker_id = worker_id
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class SignatureMethod(Enum):
    """An enumeration representing different categories.

    :cvar SHA1: "SHA1"
    :vartype SHA1: str
    :cvar SHA256: "SHA256"
    :vartype SHA256: str
    """

    SHA1 = "SHA1"
    SHA256 = "SHA256"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, SignatureMethod._member_map_.values()))


@JsonMap(
    {
        "access_token": "accessToken",
        "access_token_url": "accessTokenURL",
        "authorization_url": "authorizationURL",
        "consumer_key": "consumerKey",
        "consumer_secret": "consumerSecret",
        "request_token_url": "requestTokenURL",
        "signature_method": "signatureMethod",
        "suppress_blank_access_token": "suppressBlankAccessToken",
        "token_secret": "tokenSecret",
    }
)
class HttpoAuthSettings(BaseModel):
    """HttpoAuthSettings

    :param access_token: access_token, defaults to None
    :type access_token: str, optional
    :param access_token_url: access_token_url, defaults to None
    :type access_token_url: str, optional
    :param authorization_url: authorization_url, defaults to None
    :type authorization_url: str, optional
    :param consumer_key: consumer_key, defaults to None
    :type consumer_key: str, optional
    :param consumer_secret: consumer_secret, defaults to None
    :type consumer_secret: str, optional
    :param realm: realm, defaults to None
    :type realm: str, optional
    :param request_token_url: request_token_url, defaults to None
    :type request_token_url: str, optional
    :param signature_method: signature_method, defaults to None
    :type signature_method: SignatureMethod, optional
    :param suppress_blank_access_token: suppress_blank_access_token, defaults to None
    :type suppress_blank_access_token: bool, optional
    :param token_secret: token_secret, defaults to None
    :type token_secret: str, optional
    """

    def __init__(
        self,
        access_token: str = SENTINEL,
        access_token_url: str = SENTINEL,
        authorization_url: str = SENTINEL,
        consumer_key: str = SENTINEL,
        consumer_secret: str = SENTINEL,
        realm: str = SENTINEL,
        request_token_url: str = SENTINEL,
        signature_method: SignatureMethod = SENTINEL,
        suppress_blank_access_token: bool = SENTINEL,
        token_secret: str = SENTINEL,
        **kwargs
    ):
        """HttpoAuthSettings

        :param access_token: access_token, defaults to None
        :type access_token: str, optional
        :param access_token_url: access_token_url, defaults to None
        :type access_token_url: str, optional
        :param authorization_url: authorization_url, defaults to None
        :type authorization_url: str, optional
        :param consumer_key: consumer_key, defaults to None
        :type consumer_key: str, optional
        :param consumer_secret: consumer_secret, defaults to None
        :type consumer_secret: str, optional
        :param realm: realm, defaults to None
        :type realm: str, optional
        :param request_token_url: request_token_url, defaults to None
        :type request_token_url: str, optional
        :param signature_method: signature_method, defaults to None
        :type signature_method: SignatureMethod, optional
        :param suppress_blank_access_token: suppress_blank_access_token, defaults to None
        :type suppress_blank_access_token: bool, optional
        :param token_secret: token_secret, defaults to None
        :type token_secret: str, optional
        """
        if access_token is not SENTINEL:
            self.access_token = access_token
        if access_token_url is not SENTINEL:
            self.access_token_url = access_token_url
        if authorization_url is not SENTINEL:
            self.authorization_url = authorization_url
        if consumer_key is not SENTINEL:
            self.consumer_key = consumer_key
        if consumer_secret is not SENTINEL:
            self.consumer_secret = consumer_secret
        if realm is not SENTINEL:
            self.realm = realm
        if request_token_url is not SENTINEL:
            self.request_token_url = request_token_url
        if signature_method is not SENTINEL:
            self.signature_method = self._enum_matching(
                signature_method, SignatureMethod.list(), "signature_method"
            )
        if suppress_blank_access_token is not SENTINEL:
            self.suppress_blank_access_token = suppress_blank_access_token
        if token_secret is not SENTINEL:
            self.token_secret = token_secret
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class MapExtensionsInput(BaseModel):
    """MapExtensionsInput

    :param default: default, defaults to None
    :type default: str, optional
    :param key: key, defaults to None
    :type key: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        default: str = SENTINEL,
        key: int = SENTINEL,
        name: str = SENTINEL,
        **kwargs
    ):
        """MapExtensionsInput

        :param default: default, defaults to None
        :type default: str, optional
        :param key: key, defaults to None
        :type key: int, optional
        :param name: name, defaults to None
        :type name: str, optional
        """
        if default is not SENTINEL:
            self.default = default
        if key is not SENTINEL:
            self.key = key
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "decrypting_certificate": "decrypting-certificate",
        "session_authentication_certificate": "session-authentication-certificate",
        "signing_certificate": "signing-certificate",
        "signing_eerp_certificate": "signing-eerp-certificate",
    }
)
class OftpLocalInfo(BaseModel):
    """OftpLocalInfo

    :param decrypting_certificate: decrypting_certificate, defaults to None
    :type decrypting_certificate: str, optional
    :param session_authentication_certificate: session_authentication_certificate, defaults to None
    :type session_authentication_certificate: str, optional
    :param signing_certificate: signing_certificate, defaults to None
    :type signing_certificate: str, optional
    :param signing_eerp_certificate: signing_eerp_certificate, defaults to None
    :type signing_eerp_certificate: str, optional
    :param ssidcode: ssidcode, defaults to None
    :type ssidcode: str, optional
    :param ssidpswd: ssidpswd, defaults to None
    :type ssidpswd: str, optional
    """

    def __init__(
        self,
        decrypting_certificate: str = SENTINEL,
        session_authentication_certificate: str = SENTINEL,
        signing_certificate: str = SENTINEL,
        signing_eerp_certificate: str = SENTINEL,
        ssidcode: str = SENTINEL,
        ssidpswd: str = SENTINEL,
        **kwargs
    ):
        """OftpLocalInfo

        :param decrypting_certificate: decrypting_certificate, defaults to None
        :type decrypting_certificate: str, optional
        :param session_authentication_certificate: session_authentication_certificate, defaults to None
        :type session_authentication_certificate: str, optional
        :param signing_certificate: signing_certificate, defaults to None
        :type signing_certificate: str, optional
        :param signing_eerp_certificate: signing_eerp_certificate, defaults to None
        :type signing_eerp_certificate: str, optional
        :param ssidcode: ssidcode, defaults to None
        :type ssidcode: str, optional
        :param ssidpswd: ssidpswd, defaults to None
        :type ssidpswd: str, optional
        """
        if decrypting_certificate is not SENTINEL:
            self.decrypting_certificate = decrypting_certificate
        if session_authentication_certificate is not SENTINEL:
            self.session_authentication_certificate = session_authentication_certificate
        if signing_certificate is not SENTINEL:
            self.signing_certificate = signing_certificate
        if signing_eerp_certificate is not SENTINEL:
            self.signing_eerp_certificate = signing_eerp_certificate
        if ssidcode is not SENTINEL:
            self.ssidcode = ssidcode
        if ssidpswd is not SENTINEL:
            self.ssidpswd = ssidpswd
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .shared_web_server_cloud_tennant_general import SharedWebServerCloudTennantGeneral
from .shared_web_server_cors import SharedWebServerCors
from .shared_web_server_general import SharedWebServerGeneral
from .shared_web_server_user_management import SharedWebServerUserManagement


@JsonMap(
    {
        "atom_id": "atomId",
        "cloud_tennant_general": "cloudTennantGeneral",
        "cors_configuration": "corsConfiguration",
        "general_settings": "generalSettings",
        "should_restart_plugin": "shouldRestartPlugin",
        "user_management": "userManagement",
    }
)
class SharedWebServer(BaseModel):
    """SharedWebServer

    :param atom_id: atom_id
    :type atom_id: str
    :param cloud_tennant_general: cloud_tennant_general
    :type cloud_tennant_general: SharedWebServerCloudTennantGeneral
    :param cors_configuration: cors_configuration
    :type cors_configuration: SharedWebServerCors
    :param general_settings: general_settings
    :type general_settings: SharedWebServerGeneral
    :param should_restart_plugin: should_restart_plugin, defaults to None
    :type should_restart_plugin: bool, optional
    :param user_management: user_management
    :type user_management: SharedWebServerUserManagement
    """

    def __init__(
        self,
        atom_id: str,
        cloud_tennant_general: SharedWebServerCloudTennantGeneral,
        cors_configuration: SharedWebServerCors,
        general_settings: SharedWebServerGeneral,
        user_management: SharedWebServerUserManagement,
        should_restart_plugin: bool = SENTINEL,
        **kwargs,
    ):
        """SharedWebServer

        :param atom_id: atom_id
        :type atom_id: str
        :param cloud_tennant_general: cloud_tennant_general
        :type cloud_tennant_general: SharedWebServerCloudTennantGeneral
        :param cors_configuration: cors_configuration
        :type cors_configuration: SharedWebServerCors
        :param general_settings: general_settings
        :type general_settings: SharedWebServerGeneral
        :param should_restart_plugin: should_restart_plugin, defaults to None
        :type should_restart_plugin: bool, optional
        :param user_management: user_management
        :type user_management: SharedWebServerUserManagement
        """
        self.atom_id = atom_id
        self.cloud_tennant_general = self._define_object(
            cloud_tennant_general, SharedWebServerCloudTennantGeneral
        )
        self.cors_configuration = self._define_object(
            cors_configuration, SharedWebServerCors
        )
        self.general_settings = self._define_object(
            general_settings, SharedWebServerGeneral
        )
        if should_restart_plugin is not SENTINEL:
            self.should_restart_plugin = should_restart_plugin
        self.user_management = self._define_object(
            user_management, SharedWebServerUserManagement
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .attachment_info import AttachmentInfo
from .as2_basic_auth_info import As2BasicAuthInfo
from .public_certificate import PublicCertificate


@JsonMap(
    {
        "listen_attachment_settings": "ListenAttachmentSettings",
        "listen_auth_settings": "ListenAuthSettings",
        "as2_id": "as2Id",
        "basic_auth_enabled": "basicAuthEnabled",
        "client_ssl_certificate": "clientSSLCertificate",
        "enabled_legacy_smime": "enabledLegacySMIME",
        "encryption_public_certificate": "encryptionPublicCertificate",
        "mdn_signature_public_certificate": "mdnSignaturePublicCertificate",
        "messages_to_check_for_duplicates": "messagesToCheckForDuplicates",
        "reject_duplicate_messages": "rejectDuplicateMessages",
        "signing_public_certificate": "signingPublicCertificate",
    }
)
class As2PartnerInfo(BaseModel):
    """As2PartnerInfo

    :param listen_attachment_settings: listen_attachment_settings, defaults to None
    :type listen_attachment_settings: AttachmentInfo, optional
    :param listen_auth_settings: listen_auth_settings, defaults to None
    :type listen_auth_settings: As2BasicAuthInfo, optional
    :param as2_id: as2_id
    :type as2_id: str
    :param basic_auth_enabled: basic_auth_enabled, defaults to None
    :type basic_auth_enabled: bool, optional
    :param client_ssl_certificate: client_ssl_certificate
    :type client_ssl_certificate: PublicCertificate
    :param enabled_legacy_smime: enabled_legacy_smime, defaults to None
    :type enabled_legacy_smime: bool, optional
    :param encryption_public_certificate: encryption_public_certificate
    :type encryption_public_certificate: PublicCertificate
    :param mdn_signature_public_certificate: mdn_signature_public_certificate
    :type mdn_signature_public_certificate: PublicCertificate
    :param messages_to_check_for_duplicates: messages_to_check_for_duplicates, defaults to None
    :type messages_to_check_for_duplicates: int, optional
    :param reject_duplicate_messages: reject_duplicate_messages, defaults to None
    :type reject_duplicate_messages: bool, optional
    :param signing_public_certificate: signing_public_certificate
    :type signing_public_certificate: PublicCertificate
    """

    def __init__(
        self,
        as2_id: str,
        client_ssl_certificate: PublicCertificate,
        encryption_public_certificate: PublicCertificate,
        mdn_signature_public_certificate: PublicCertificate,
        signing_public_certificate: PublicCertificate,
        listen_attachment_settings: AttachmentInfo = SENTINEL,
        listen_auth_settings: As2BasicAuthInfo = SENTINEL,
        basic_auth_enabled: bool = SENTINEL,
        enabled_legacy_smime: bool = SENTINEL,
        messages_to_check_for_duplicates: int = SENTINEL,
        reject_duplicate_messages: bool = SENTINEL,
        **kwargs,
    ):
        """As2PartnerInfo

        :param listen_attachment_settings: listen_attachment_settings, defaults to None
        :type listen_attachment_settings: AttachmentInfo, optional
        :param listen_auth_settings: listen_auth_settings, defaults to None
        :type listen_auth_settings: As2BasicAuthInfo, optional
        :param as2_id: as2_id
        :type as2_id: str
        :param basic_auth_enabled: basic_auth_enabled, defaults to None
        :type basic_auth_enabled: bool, optional
        :param client_ssl_certificate: client_ssl_certificate
        :type client_ssl_certificate: PublicCertificate
        :param enabled_legacy_smime: enabled_legacy_smime, defaults to None
        :type enabled_legacy_smime: bool, optional
        :param encryption_public_certificate: encryption_public_certificate
        :type encryption_public_certificate: PublicCertificate
        :param mdn_signature_public_certificate: mdn_signature_public_certificate
        :type mdn_signature_public_certificate: PublicCertificate
        :param messages_to_check_for_duplicates: messages_to_check_for_duplicates, defaults to None
        :type messages_to_check_for_duplicates: int, optional
        :param reject_duplicate_messages: reject_duplicate_messages, defaults to None
        :type reject_duplicate_messages: bool, optional
        :param signing_public_certificate: signing_public_certificate
        :type signing_public_certificate: PublicCertificate
        """
        if listen_attachment_settings is not SENTINEL:
            self.listen_attachment_settings = self._define_object(
                listen_attachment_settings, AttachmentInfo
            )
        if listen_auth_settings is not SENTINEL:
            self.listen_auth_settings = self._define_object(
                listen_auth_settings, As2BasicAuthInfo
            )
        self.as2_id = as2_id
        if basic_auth_enabled is not SENTINEL:
            self.basic_auth_enabled = basic_auth_enabled
        self.client_ssl_certificate = self._define_object(
            client_ssl_certificate, PublicCertificate
        )
        if enabled_legacy_smime is not SENTINEL:
            self.enabled_legacy_smime = enabled_legacy_smime
        self.encryption_public_certificate = self._define_object(
            encryption_public_certificate, PublicCertificate
        )
        self.mdn_signature_public_certificate = self._define_object(
            mdn_signature_public_certificate, PublicCertificate
        )
        if messages_to_check_for_duplicates is not SENTINEL:
            self.messages_to_check_for_duplicates = messages_to_check_for_duplicates
        if reject_duplicate_messages is not SENTINEL:
            self.reject_duplicate_messages = reject_duplicate_messages
        self.signing_public_certificate = self._define_object(
            signing_public_certificate, PublicCertificate
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"atom_id": "atomId", "environment_id": "environmentId", "id_": "id"})
class EnvironmentAtomAttachment(BaseModel):
    """EnvironmentAtomAttachment

    :param atom_id: The ID of the Runtime., defaults to None
    :type atom_id: str, optional
    :param environment_id: The ID of the environment., defaults to None
    :type environment_id: str, optional
    :param id_: The object’s conceptual ID, which is synthesized from the Runtime and environment IDs., defaults to None
    :type id_: str, optional
    """

    def __init__(
        self,
        atom_id: str = SENTINEL,
        environment_id: str = SENTINEL,
        id_: str = SENTINEL,
        **kwargs
    ):
        """EnvironmentAtomAttachment

        :param atom_id: The ID of the Runtime., defaults to None
        :type atom_id: str, optional
        :param environment_id: The ID of the environment., defaults to None
        :type environment_id: str, optional
        :param id_: The object’s conceptual ID, which is synthesized from the Runtime and environment IDs., defaults to None
        :type id_: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if environment_id is not SENTINEL:
            self.environment_id = environment_id
        if id_ is not SENTINEL:
            self.id_ = id_
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .process_integration_pack_info import ProcessIntegrationPackInfo


@JsonMap(
    {"description": "Description", "integration_pack": "IntegrationPack", "id_": "id"}
)
class Process(BaseModel):
    """Process

    :param description: description, defaults to None
    :type description: str, optional
    :param integration_pack: integration_pack, defaults to None
    :type integration_pack: List[ProcessIntegrationPackInfo], optional
    :param id_: A unique ID assigned by the system to the process. For deployed processes and processes belonging to single-install integration packs, this value is the process component ID.For processes belonging to multi-install integration packs, this is an synthetic ID and does not match an actual process component. You can use this value as the extensionGroupId when querying the Environment Extensions object, defaults to None
    :type id_: str, optional
    :param name: The name of the process., defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        description: str = SENTINEL,
        integration_pack: List[ProcessIntegrationPackInfo] = SENTINEL,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        **kwargs,
    ):
        """Process

        :param description: description, defaults to None
        :type description: str, optional
        :param integration_pack: integration_pack, defaults to None
        :type integration_pack: List[ProcessIntegrationPackInfo], optional
        :param id_: A unique ID assigned by the system to the process. For deployed processes and processes belonging to single-install integration packs, this value is the process component ID.For processes belonging to multi-install integration packs, this is an synthetic ID and does not match an actual process component. You can use this value as the extensionGroupId when querying the Environment Extensions object, defaults to None
        :type id_: str, optional
        :param name: The name of the process., defaults to None
        :type name: str, optional
        """
        if description is not SENTINEL:
            self.description = description
        if integration_pack is not SENTINEL:
            self.integration_pack = self._define_list(
                integration_pack, ProcessIntegrationPackInfo
            )
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from typing import Awaitable
from .utils.to_async import to_async
from ..execute_process import ExecuteProcessService


class ExecuteProcessServiceAsync(ExecuteProcessService):
    """
    Async Wrapper for ExecuteProcessServiceAsync
    """

    def create_execute_process(self) -> Awaitable[None]:
        return to_async(super().create_execute_process)()




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    ComponentReference,
    ComponentReferenceBulkRequest,
    ComponentReferenceBulkResponse,
    ComponentReferenceQueryConfig,
    ComponentReferenceQueryResponse,
)


class ComponentReferenceService(BaseService):

    @cast_models
    def get_component_reference(
        self, component_id: str
    ) -> Union[ComponentReference, str]:
        """Retrieves the component reference for a component ID.

         Send an HTTP GET to `https://api.boomi.com/api/rest/v1/{accountId}/ComponentReference/{componentId}`

        where `{accountId}` is the ID of the authenticating account for the request and `{componentId}` is the ID of the secondary component whose references you are attempting to GET.

        If you want to specify a branch, send an HTTP GET to `https://api.boomi.com/api/rest/v1/{accountId}/ComponentReference/{componentId}~{branchId}`

        where `{accountId}` is the ID of the authenticating account for the request and `{componentId}` is the ID of the secondary component whose references you are attempting to GET, and `{branchId}` is the branch on which you want to GET component references.

        :param component_id: The ID of the secondary component. The component ID is available in the **Revision History** dialog, which you can access from the **Build** page in the service.
        :type component_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentReference, str]
        """

        Validator(str).validate(component_id)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentReference/{{componentId}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("componentId", component_id)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentReference._unmap(response)
        if content == "application/xml":
            return ComponentReference._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_component_reference(
        self, request_body: ComponentReferenceBulkRequest = None
    ) -> Union[ComponentReferenceBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: ComponentReferenceBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentReferenceBulkResponse, str]
        """

        Validator(ComponentReferenceBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentReference/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentReferenceBulkResponse._unmap(response)
        if content == "application/xml":
            return ComponentReferenceBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_component_reference(
        self, request_body: ComponentReferenceQueryConfig = None
    ) -> Union[ComponentReferenceQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        - You can use the QUERY operation to return the latest version(s) of a primary component(s) that references a given secondary component ID, or all the secondary components that the given primary component ID references.

        >**Note:** When querying either primary or secondary component references, the API object returns the immediate reference (one level). It does not recursively trace through nested references like the **Show Where Used** feature does in the user interface.

         For example, take a process that references a Map component where it references two Profile components. If you query by `parentComponentId=<process>`, the API returns a result for the Map component but not the profiles. To get the profiles, you need to perform a second call to query references for the Map component.

        - You can filter the query operation in one of two ways:

          - To find all the secondary components referenced by a given primary component, you must provide both the parentComponentId and the parentVersion values. You can optionally use the type filter in your query.

          - To find all the primary components that reference a given secondary component, you must provide the componentId value. You can optionally include the type filter in your query.

        - To see more information about a component ID returned in the response, like the component's type or name, you can query that same ID using the [Component Metadata object](/api/platformapi#tag/ComponentMetadata).

         #### Understanding references to deleted components

         Filtering or querying by `componentId` only returns the component's current version. If you delete the current component revision, it does not return results.

        When filtering by `parentComponentId` or `parentVersion`, it saves references to other components for a given version of the primary component. If you delete the given primary component version, it does not return results. Note that it is possible to return a reference to a deleted secondary component if you do not remove the reference in the user interface (appears in red).

        :param request_body: The request body., defaults to None
        :type request_body: ComponentReferenceQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentReferenceQueryResponse, str]
        """

        Validator(ComponentReferenceQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentReference/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentReferenceQueryResponse._unmap(response)
        if content == "application/xml":
            return ComponentReferenceQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_component_reference(
        self, request_body: str
    ) -> Union[ComponentReferenceQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentReferenceQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentReference/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentReferenceQueryResponse._unmap(response)
        if content == "application/xml":
            return ComponentReferenceQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class FilterMatchType(Enum):
    """An enumeration representing different categories.

    :cvar WILDCARD: "wildcard"
    :vartype WILDCARD: str
    :cvar REGEX: "regex"
    :vartype REGEX: str
    """

    WILDCARD = "wildcard"
    REGEX = "regex"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, FilterMatchType._member_map_.values()))


@JsonMap(
    {
        "delete_after_read": "deleteAfterRead",
        "file_filter": "fileFilter",
        "filter_match_type": "filterMatchType",
        "get_directory": "getDirectory",
        "max_file_count": "maxFileCount",
        "use_default_get_options": "useDefaultGetOptions",
    }
)
class DiskGetOptions(BaseModel):
    """DiskGetOptions

    :param delete_after_read: delete_after_read, defaults to None
    :type delete_after_read: bool, optional
    :param file_filter: file_filter
    :type file_filter: str
    :param filter_match_type: filter_match_type, defaults to None
    :type filter_match_type: FilterMatchType, optional
    :param get_directory: get_directory
    :type get_directory: str
    :param max_file_count: max_file_count, defaults to None
    :type max_file_count: int, optional
    :param use_default_get_options: use_default_get_options, defaults to None
    :type use_default_get_options: bool, optional
    """

    def __init__(
        self,
        file_filter: str,
        get_directory: str,
        delete_after_read: bool = SENTINEL,
        filter_match_type: FilterMatchType = SENTINEL,
        max_file_count: int = SENTINEL,
        use_default_get_options: bool = SENTINEL,
        **kwargs
    ):
        """DiskGetOptions

        :param delete_after_read: delete_after_read, defaults to None
        :type delete_after_read: bool, optional
        :param file_filter: file_filter
        :type file_filter: str
        :param filter_match_type: filter_match_type, defaults to None
        :type filter_match_type: FilterMatchType, optional
        :param get_directory: get_directory
        :type get_directory: str
        :param max_file_count: max_file_count, defaults to None
        :type max_file_count: int, optional
        :param use_default_get_options: use_default_get_options, defaults to None
        :type use_default_get_options: bool, optional
        """
        if delete_after_read is not SENTINEL:
            self.delete_after_read = delete_after_read
        self.file_filter = file_filter
        if filter_match_type is not SENTINEL:
            self.filter_match_type = self._enum_matching(
                filter_match_type, FilterMatchType.list(), "filter_match_type"
            )
        self.get_directory = get_directory
        if max_file_count is not SENTINEL:
            self.max_file_count = max_file_count
        if use_default_get_options is not SENTINEL:
            self.use_default_get_options = use_default_get_options
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ApiUsageCountClassification(Enum):
    """An enumeration representing different categories.

    :cvar PROD: "PROD"
    :vartype PROD: str
    :cvar TEST: "TEST"
    :vartype TEST: str
    """

    PROD = "PROD"
    TEST = "TEST"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, ApiUsageCountClassification._member_map_.values())
        )


@JsonMap(
    {
        "error_count": "errorCount",
        "process_date": "processDate",
        "success_count": "successCount",
    }
)
class ApiUsageCount(BaseModel):
    """ApiUsageCount

    :param classification: The environment classification., defaults to None
    :type classification: ApiUsageCountClassification, optional
    :param error_count: The count of unsuccessful process runs, where the status is error or aborted., defaults to None
    :type error_count: int, optional
    :param process_date: The start time of the day, in UTC.   Date with the format yyyy-MM-dd'T'HH:mm:ss'Z' — for example, 2017-09-01T00:00:00Z.To specify a time block, use the BETWEEN operator with two arguments, one representing the start time and the other representing the end time.   Boomi recommends specifying a time block in all queries, particularly for heavy users of Low Latency processes, as a means of preventing the return of excessively large amounts of data., defaults to None
    :type process_date: str, optional
    :param success_count: The count of successful process runs, where a successful run is one with a status of complete., defaults to None
    :type success_count: int, optional
    """

    def __init__(
        self,
        classification: ApiUsageCountClassification = SENTINEL,
        error_count: int = SENTINEL,
        process_date: str = SENTINEL,
        success_count: int = SENTINEL,
        **kwargs
    ):
        """ApiUsageCount

        :param classification: The environment classification., defaults to None
        :type classification: ApiUsageCountClassification, optional
        :param error_count: The count of unsuccessful process runs, where the status is error or aborted., defaults to None
        :type error_count: int, optional
        :param process_date: The start time of the day, in UTC.   Date with the format yyyy-MM-dd'T'HH:mm:ss'Z' — for example, 2017-09-01T00:00:00Z.To specify a time block, use the BETWEEN operator with two arguments, one representing the start time and the other representing the end time.   Boomi recommends specifying a time block in all queries, particularly for heavy users of Low Latency processes, as a means of preventing the return of excessively large amounts of data., defaults to None
        :type process_date: str, optional
        :param success_count: The count of successful process runs, where a successful run is one with a status of complete., defaults to None
        :type success_count: int, optional
        """
        if classification is not SENTINEL:
            self.classification = self._enum_matching(
                classification, ApiUsageCountClassification.list(), "classification"
            )
        if error_count is not SENTINEL:
            self.error_count = error_count
        if process_date is not SENTINEL:
            self.process_date = process_date
        if success_count is not SENTINEL:
            self.success_count = success_count
        self._kwargs = kwargs




from typing import Optional


class RequestError(Exception):
    """
    Class representing a Request Error.
    """

    def __init__(
        self,
        message: str,
        stack: Optional["RequestError"] = None,
    ):
        """
        Initialize a new instance of RequestError.

        :param str message: The error message.
        """
        super().__init__(message)
        self.stack = stack

    def __str__(self):
        """
        Get the string representation of the error.

        :return: The string representation of the error.
        :rtype: str
        """
        error_stack = []
        current_error = self
        while current_error is not None:
            error_stack.append(f"Error: {super().__str__()}")
            current_error = current_error.stack
        return "\n".join(error_stack)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .account_user_role_expression import (
    AccountUserRoleExpression,
    AccountUserRoleExpressionGuard,
)
from .account_user_role_simple_expression import AccountUserRoleSimpleExpression
from .account_user_role_grouping_expression import AccountUserRoleGroupingExpression


@JsonMap({})
class AccountUserRoleQueryConfigQueryFilter(BaseModel):
    """AccountUserRoleQueryConfigQueryFilter

    :param expression: expression
    :type expression: AccountUserRoleExpression
    """

    def __init__(self, expression: AccountUserRoleExpression, **kwargs):
        """AccountUserRoleQueryConfigQueryFilter

        :param expression: expression
        :type expression: AccountUserRoleExpression
        """
        self.expression = AccountUserRoleExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AccountUserRoleQueryConfig(BaseModel):
    """AccountUserRoleQueryConfig

    :param query_filter: query_filter
    :type query_filter: AccountUserRoleQueryConfigQueryFilter
    """

    def __init__(self, query_filter: AccountUserRoleQueryConfigQueryFilter, **kwargs):
        """AccountUserRoleQueryConfig

        :param query_filter: query_filter
        :type query_filter: AccountUserRoleQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, AccountUserRoleQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .edifact_connector_record_simple_expression import (
    EdifactConnectorRecordSimpleExpression,
)
from .edifact_connector_record_grouping_expression import (
    EdifactConnectorRecordGroupingExpression,
)


class EdifactConnectorRecordExpressionGuard(OneOfBaseModel):
    class_list = {
        "EdifactConnectorRecordSimpleExpression": EdifactConnectorRecordSimpleExpression,
        "EdifactConnectorRecordGroupingExpression": EdifactConnectorRecordGroupingExpression,
    }


EdifactConnectorRecordExpression = Union[
    EdifactConnectorRecordSimpleExpression, EdifactConnectorRecordGroupingExpression
]




from typing import Dict

from .base_header import BaseHeader


class AccessTokenAuth(BaseHeader):
    """
    A class for handling Access token authentication in headers.

    :ivar str _token_prefix: The prefix for the token in the header.
    :ivar str token_value: The value of the token.
    """

    _token_prefix = "Bearer"

    def __init__(self, token_value: str):
        """
        Initialize the AccessTokenAuth instance.

        :param token_value: The value of the token.
        :type token_value: str
        """
        self.token_value = token_value

    def set_value(self, value: str) -> None:
        """
        Set the value of the token.

        :param value: The new value of the token.
        :type value: str
        """
        self.token_value = value

    def get_headers(self) -> Dict[str, str]:
        """
        Get the headers with the Authorization field set to the Access token.

        :return: A dictionary with the Authorization field set to the Access token.
        :rtype: Dict[str, str]
        """
        return {"Authorization": f"{self._token_prefix} {self.token_value}"}




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .edi_delimiter import EdiDelimiter
from .edi_segment_terminator import EdiSegmentTerminator


class Acceptackoption(Enum):
    """An enumeration representing different categories.

    :cvar AL: "AL"
    :vartype AL: str
    :cvar NE: "NE"
    :vartype NE: str
    :cvar ER: "ER"
    :vartype ER: str
    :cvar SU: "SU"
    :vartype SU: str
    :cvar NOTDEFINED: "NOT_DEFINED"
    :vartype NOTDEFINED: str
    """

    AL = "AL"
    NE = "NE"
    ER = "ER"
    SU = "SU"
    NOTDEFINED = "NOT_DEFINED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Acceptackoption._member_map_.values()))


class Appackoption(Enum):
    """An enumeration representing different categories.

    :cvar AL: "AL"
    :vartype AL: str
    :cvar NE: "NE"
    :vartype NE: str
    :cvar ER: "ER"
    :vartype ER: str
    :cvar SU: "SU"
    :vartype SU: str
    :cvar NOTDEFINED: "NOT_DEFINED"
    :vartype NOTDEFINED: str
    """

    AL = "AL"
    NE = "NE"
    ER = "ER"
    SU = "SU"
    NOTDEFINED = "NOT_DEFINED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Appackoption._member_map_.values()))


class Batchoption(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "none"
    :vartype NONE: str
    :cvar BATCH: "batch"
    :vartype BATCH: str
    """

    NONE = "none"
    BATCH = "batch"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Batchoption._member_map_.values()))


class Hl7OptionsOutboundValidationOption(Enum):
    """An enumeration representing different categories.

    :cvar FILTERERROR: "filterError"
    :vartype FILTERERROR: str
    :cvar FAILALL: "failAll"
    :vartype FAILALL: str
    """

    FILTERERROR = "filterError"
    FAILALL = "failAll"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                Hl7OptionsOutboundValidationOption._member_map_.values(),
            )
        )


@JsonMap(
    {
        "composite_delimiter": "compositeDelimiter",
        "element_delimiter": "elementDelimiter",
        "outbound_interchange_validation": "outboundInterchangeValidation",
        "outbound_validation_option": "outboundValidationOption",
        "reject_duplicates": "rejectDuplicates",
        "segment_terminator": "segmentTerminator",
        "sub_composite_delimiter": "subCompositeDelimiter",
    }
)
class Hl7Options(BaseModel):
    """Hl7Options

    :param acceptackoption: acceptackoption, defaults to None
    :type acceptackoption: Acceptackoption, optional
    :param appackoption: appackoption, defaults to None
    :type appackoption: Appackoption, optional
    :param batchoption: batchoption, defaults to None
    :type batchoption: Batchoption, optional
    :param composite_delimiter: composite_delimiter
    :type composite_delimiter: EdiDelimiter
    :param element_delimiter: element_delimiter
    :type element_delimiter: EdiDelimiter
    :param filteracknowledgements: filteracknowledgements, defaults to None
    :type filteracknowledgements: bool, optional
    :param outbound_interchange_validation: outbound_interchange_validation, defaults to None
    :type outbound_interchange_validation: bool, optional
    :param outbound_validation_option: outbound_validation_option, defaults to None
    :type outbound_validation_option: Hl7OptionsOutboundValidationOption, optional
    :param reject_duplicates: reject_duplicates, defaults to None
    :type reject_duplicates: bool, optional
    :param segment_terminator: segment_terminator
    :type segment_terminator: EdiSegmentTerminator
    :param sub_composite_delimiter: sub_composite_delimiter
    :type sub_composite_delimiter: EdiDelimiter
    """

    def __init__(
        self,
        composite_delimiter: EdiDelimiter,
        element_delimiter: EdiDelimiter,
        segment_terminator: EdiSegmentTerminator,
        sub_composite_delimiter: EdiDelimiter,
        acceptackoption: Acceptackoption = SENTINEL,
        appackoption: Appackoption = SENTINEL,
        batchoption: Batchoption = SENTINEL,
        filteracknowledgements: bool = SENTINEL,
        outbound_interchange_validation: bool = SENTINEL,
        outbound_validation_option: Hl7OptionsOutboundValidationOption = SENTINEL,
        reject_duplicates: bool = SENTINEL,
        **kwargs,
    ):
        """Hl7Options

        :param acceptackoption: acceptackoption, defaults to None
        :type acceptackoption: Acceptackoption, optional
        :param appackoption: appackoption, defaults to None
        :type appackoption: Appackoption, optional
        :param batchoption: batchoption, defaults to None
        :type batchoption: Batchoption, optional
        :param composite_delimiter: composite_delimiter
        :type composite_delimiter: EdiDelimiter
        :param element_delimiter: element_delimiter
        :type element_delimiter: EdiDelimiter
        :param filteracknowledgements: filteracknowledgements, defaults to None
        :type filteracknowledgements: bool, optional
        :param outbound_interchange_validation: outbound_interchange_validation, defaults to None
        :type outbound_interchange_validation: bool, optional
        :param outbound_validation_option: outbound_validation_option, defaults to None
        :type outbound_validation_option: Hl7OptionsOutboundValidationOption, optional
        :param reject_duplicates: reject_duplicates, defaults to None
        :type reject_duplicates: bool, optional
        :param segment_terminator: segment_terminator
        :type segment_terminator: EdiSegmentTerminator
        :param sub_composite_delimiter: sub_composite_delimiter
        :type sub_composite_delimiter: EdiDelimiter
        """
        if acceptackoption is not SENTINEL:
            self.acceptackoption = self._enum_matching(
                acceptackoption, Acceptackoption.list(), "acceptackoption"
            )
        if appackoption is not SENTINEL:
            self.appackoption = self._enum_matching(
                appackoption, Appackoption.list(), "appackoption"
            )
        if batchoption is not SENTINEL:
            self.batchoption = self._enum_matching(
                batchoption, Batchoption.list(), "batchoption"
            )
        self.composite_delimiter = self._define_object(
            composite_delimiter, EdiDelimiter
        )
        self.element_delimiter = self._define_object(element_delimiter, EdiDelimiter)
        if filteracknowledgements is not SENTINEL:
            self.filteracknowledgements = filteracknowledgements
        if outbound_interchange_validation is not SENTINEL:
            self.outbound_interchange_validation = outbound_interchange_validation
        if outbound_validation_option is not SENTINEL:
            self.outbound_validation_option = self._enum_matching(
                outbound_validation_option,
                Hl7OptionsOutboundValidationOption.list(),
                "outbound_validation_option",
            )
        if reject_duplicates is not SENTINEL:
            self.reject_duplicates = reject_duplicates
        self.segment_terminator = self._define_object(
            segment_terminator, EdiSegmentTerminator
        )
        self.sub_composite_delimiter = self._define_object(
            sub_composite_delimiter, EdiDelimiter
        )
        self._kwargs = kwargs




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class EventSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    :cvar LIKE: "LIKE"
    :vartype LIKE: str
    :cvar NOTEQUALS: "NOT_EQUALS"
    :vartype NOTEQUALS: str
    :cvar ISNULL: "IS_NULL"
    :vartype ISNULL: str
    :cvar ISNOTNULL: "IS_NOT_NULL"
    :vartype ISNOTNULL: str
    :cvar BETWEEN: "BETWEEN"
    :vartype BETWEEN: str
    :cvar GREATERTHAN: "GREATER_THAN"
    :vartype GREATERTHAN: str
    :cvar GREATERTHANOREQUAL: "GREATER_THAN_OR_EQUAL"
    :vartype GREATERTHANOREQUAL: str
    :cvar LESSTHAN: "LESS_THAN"
    :vartype LESSTHAN: str
    :cvar LESSTHANOREQUAL: "LESS_THAN_OR_EQUAL"
    :vartype LESSTHANOREQUAL: str
    :cvar CONTAINS: "CONTAINS"
    :vartype CONTAINS: str
    :cvar NOTCONTAINS: "NOT_CONTAINS"
    :vartype NOTCONTAINS: str
    """

    EQUALS = "EQUALS"
    LIKE = "LIKE"
    NOTEQUALS = "NOT_EQUALS"
    ISNULL = "IS_NULL"
    ISNOTNULL = "IS_NOT_NULL"
    BETWEEN = "BETWEEN"
    GREATERTHAN = "GREATER_THAN"
    GREATERTHANOREQUAL = "GREATER_THAN_OR_EQUAL"
    LESSTHAN = "LESS_THAN"
    LESSTHANOREQUAL = "LESS_THAN_OR_EQUAL"
    CONTAINS = "CONTAINS"
    NOTCONTAINS = "NOT_CONTAINS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, EventSimpleExpressionOperator._member_map_.values())
        )


class EventSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar EVENTID: "eventId"
    :vartype EVENTID: str
    :cvar ACCOUNTID: "accountId"
    :vartype ACCOUNTID: str
    :cvar ATOMID: "atomId"
    :vartype ATOMID: str
    :cvar ATOMNAME: "atomName"
    :vartype ATOMNAME: str
    :cvar EVENTLEVEL: "eventLevel"
    :vartype EVENTLEVEL: str
    :cvar EVENTDATE: "eventDate"
    :vartype EVENTDATE: str
    :cvar STATUS: "status"
    :vartype STATUS: str
    :cvar EVENTTYPE: "eventType"
    :vartype EVENTTYPE: str
    :cvar EXECUTIONID: "executionId"
    :vartype EXECUTIONID: str
    :cvar TITLE: "title"
    :vartype TITLE: str
    :cvar UPDATEDATE: "updateDate"
    :vartype UPDATEDATE: str
    :cvar STARTTIME: "startTime"
    :vartype STARTTIME: str
    :cvar ENDTIME: "endTime"
    :vartype ENDTIME: str
    :cvar ERRORDOCUMENTCOUNT: "errorDocumentCount"
    :vartype ERRORDOCUMENTCOUNT: str
    :cvar INBOUNDDOCUMENTCOUNT: "inboundDocumentCount"
    :vartype INBOUNDDOCUMENTCOUNT: str
    :cvar OUTBOUNDDOCUMENTCOUNT: "outboundDocumentCount"
    :vartype OUTBOUNDDOCUMENTCOUNT: str
    :cvar PROCESSNAME: "processName"
    :vartype PROCESSNAME: str
    :cvar RECORDDATE: "recordDate"
    :vartype RECORDDATE: str
    :cvar ERROR: "error"
    :vartype ERROR: str
    :cvar ENVIRONMENT: "environment"
    :vartype ENVIRONMENT: str
    :cvar CLASSIFICATION: "classification"
    :vartype CLASSIFICATION: str
    """

    EVENTID = "eventId"
    ACCOUNTID = "accountId"
    ATOMID = "atomId"
    ATOMNAME = "atomName"
    EVENTLEVEL = "eventLevel"
    EVENTDATE = "eventDate"
    STATUS = "status"
    EVENTTYPE = "eventType"
    EXECUTIONID = "executionId"
    TITLE = "title"
    UPDATEDATE = "updateDate"
    STARTTIME = "startTime"
    ENDTIME = "endTime"
    ERRORDOCUMENTCOUNT = "errorDocumentCount"
    INBOUNDDOCUMENTCOUNT = "inboundDocumentCount"
    OUTBOUNDDOCUMENTCOUNT = "outboundDocumentCount"
    PROCESSNAME = "processName"
    RECORDDATE = "recordDate"
    ERROR = "error"
    ENVIRONMENT = "environment"
    CLASSIFICATION = "classification"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, EventSimpleExpressionProperty._member_map_.values())
        )


@JsonMap({})
class EventSimpleExpression(BaseModel):
    """EventSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: EventSimpleExpressionOperator
    :param property: property
    :type property: EventSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: EventSimpleExpressionOperator,
        property: EventSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """EventSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: EventSimpleExpressionOperator
        :param property: property
        :type property: EventSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator, EventSimpleExpressionOperator.list(), "operator"
        )
        self.property = self._enum_matching(
            property, EventSimpleExpressionProperty.list(), "property"
        )
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ProcessingId(Enum):
    """An enumeration representing different categories.

    :cvar D: "D"
    :vartype D: str
    :cvar P: "P"
    :vartype P: str
    :cvar T: "T"
    :vartype T: str
    """

    D = "D"
    P = "P"
    T = "T"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ProcessingId._member_map_.values()))


class ProcessingMode(Enum):
    """An enumeration representing different categories.

    :cvar A: "A"
    :vartype A: str
    :cvar R: "R"
    :vartype R: str
    :cvar I: "I"
    :vartype I: str
    :cvar T: "T"
    :vartype T: str
    :cvar NOTPRESENT: "NOT_PRESENT"
    :vartype NOTPRESENT: str
    """

    A = "A"
    R = "R"
    I = "I"
    T = "T"
    NOTPRESENT = "NOT_PRESENT"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ProcessingMode._member_map_.values()))


@JsonMap({"processing_id": "processingId", "processing_mode": "processingMode"})
class ProcessingType(BaseModel):
    """ProcessingType

    :param processing_id: processing_id, defaults to None
    :type processing_id: ProcessingId, optional
    :param processing_mode: processing_mode, defaults to None
    :type processing_mode: ProcessingMode, optional
    """

    def __init__(
        self,
        processing_id: ProcessingId = SENTINEL,
        processing_mode: ProcessingMode = SENTINEL,
        **kwargs
    ):
        """ProcessingType

        :param processing_id: processing_id, defaults to None
        :type processing_id: ProcessingId, optional
        :param processing_mode: processing_mode, defaults to None
        :type processing_mode: ProcessingMode, optional
        """
        if processing_id is not SENTINEL:
            self.processing_id = self._enum_matching(
                processing_id, ProcessingId.list(), "processing_id"
            )
        if processing_mode is not SENTINEL:
            self.processing_mode = self._enum_matching(
                processing_mode, ProcessingMode.list(), "processing_mode"
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account import AccountService
from ...models import (
    Account,
    AccountBulkResponse,
    AccountBulkRequest,
    AccountQueryResponse,
    AccountQueryConfig,
)


class AccountServiceAsync(AccountService):
    """
    Async Wrapper for AccountServiceAsync
    """

    def get_account(self, id_: str) -> Awaitable[Union[Account, str]]:
        return to_async(super().get_account)(id_)

    def bulk_account(
        self, request_body: AccountBulkRequest = None
    ) -> Awaitable[Union[AccountBulkResponse, str]]:
        return to_async(super().bulk_account)(request_body)

    def query_account(
        self, request_body: AccountQueryConfig = None
    ) -> Awaitable[Union[AccountQueryResponse, str]]:
        return to_async(super().query_account)(request_body)

    def query_more_account(
        self, request_body: str
    ) -> Awaitable[Union[AccountQueryResponse, str]]:
        return to_async(super().query_more_account)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .atom_expression import AtomExpression, AtomExpressionGuard
from .atom_simple_expression import AtomSimpleExpression
from .atom_grouping_expression import AtomGroupingExpression


@JsonMap({})
class AtomQueryConfigQueryFilter(BaseModel):
    """AtomQueryConfigQueryFilter

    :param expression: expression
    :type expression: AtomExpression
    """

    def __init__(self, expression: AtomExpression, **kwargs):
        """AtomQueryConfigQueryFilter

        :param expression: expression
        :type expression: AtomExpression
        """
        self.expression = AtomExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AtomQueryConfig(BaseModel):
    """AtomQueryConfig

    :param query_filter: query_filter (optional)
    :type query_filter: AtomQueryConfigQueryFilter
    """

    def __init__(self, query_filter: AtomQueryConfigQueryFilter = None, **kwargs):
        """AtomQueryConfig

        :param query_filter: query_filter (optional)
        :type query_filter: AtomQueryConfigQueryFilter
        """
        if query_filter is not None:
            self.query_filter = self._define_object(
                query_filter, AtomQueryConfigQueryFilter
            )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .environment_map_extension_external_component_expression import (
    EnvironmentMapExtensionExternalComponentExpression,
    EnvironmentMapExtensionExternalComponentExpressionGuard,
)
from .environment_map_extension_external_component_simple_expression import (
    EnvironmentMapExtensionExternalComponentSimpleExpression,
)
from .environment_map_extension_external_component_grouping_expression import (
    EnvironmentMapExtensionExternalComponentGroupingExpression,
)


@JsonMap({})
class EnvironmentMapExtensionExternalComponentQueryConfigQueryFilter(BaseModel):
    """EnvironmentMapExtensionExternalComponentQueryConfigQueryFilter

    :param expression: expression
    :type expression: EnvironmentMapExtensionExternalComponentExpression
    """

    def __init__(
        self, expression: EnvironmentMapExtensionExternalComponentExpression, **kwargs
    ):
        """EnvironmentMapExtensionExternalComponentQueryConfigQueryFilter

        :param expression: expression
        :type expression: EnvironmentMapExtensionExternalComponentExpression
        """
        self.expression = (
            EnvironmentMapExtensionExternalComponentExpressionGuard.return_one_of(
                expression
            )
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EnvironmentMapExtensionExternalComponentQueryConfig(BaseModel):
    """EnvironmentMapExtensionExternalComponentQueryConfig

    :param query_filter: query_filter
    :type query_filter: EnvironmentMapExtensionExternalComponentQueryConfigQueryFilter
    """

    def __init__(
        self,
        query_filter: EnvironmentMapExtensionExternalComponentQueryConfigQueryFilter,
        **kwargs,
    ):
        """EnvironmentMapExtensionExternalComponentQueryConfig

        :param query_filter: query_filter
        :type query_filter: EnvironmentMapExtensionExternalComponentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, EnvironmentMapExtensionExternalComponentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "atom_id": "atomId",
        "component_id": "componentId",
        "component_type": "componentType",
        "id_": "id",
    }
)
class ComponentAtomAttachment(BaseModel):
    """ComponentAtomAttachment

    :param atom_id: atom_id, defaults to None
    :type atom_id: str, optional
    :param component_id: component_id, defaults to None
    :type component_id: str, optional
    :param component_type: component_type, defaults to None
    :type component_type: str, optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    """

    def __init__(
        self,
        atom_id: str = SENTINEL,
        component_id: str = SENTINEL,
        component_type: str = SENTINEL,
        id_: str = SENTINEL,
        **kwargs
    ):
        """ComponentAtomAttachment

        :param atom_id: atom_id, defaults to None
        :type atom_id: str, optional
        :param component_id: component_id, defaults to None
        :type component_id: str, optional
        :param component_type: component_type, defaults to None
        :type component_type: str, optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if component_id is not SENTINEL:
            self.component_id = component_id
        if component_type is not SENTINEL:
            self.component_type = component_type
        if id_ is not SENTINEL:
            self.id_ = id_
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .integration_pack_atom_attachment_expression import (
    IntegrationPackAtomAttachmentExpression,
    IntegrationPackAtomAttachmentExpressionGuard,
)
from .integration_pack_atom_attachment_simple_expression import (
    IntegrationPackAtomAttachmentSimpleExpression,
)
from .integration_pack_atom_attachment_grouping_expression import (
    IntegrationPackAtomAttachmentGroupingExpression,
)


@JsonMap({})
class IntegrationPackAtomAttachmentQueryConfigQueryFilter(BaseModel):
    """IntegrationPackAtomAttachmentQueryConfigQueryFilter

    :param expression: expression
    :type expression: IntegrationPackAtomAttachmentExpression
    """

    def __init__(self, expression: IntegrationPackAtomAttachmentExpression, **kwargs):
        """IntegrationPackAtomAttachmentQueryConfigQueryFilter

        :param expression: expression
        :type expression: IntegrationPackAtomAttachmentExpression
        """
        self.expression = IntegrationPackAtomAttachmentExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class IntegrationPackAtomAttachmentQueryConfig(BaseModel):
    """IntegrationPackAtomAttachmentQueryConfig

    :param query_filter: query_filter
    :type query_filter: IntegrationPackAtomAttachmentQueryConfigQueryFilter
    """

    def __init__(
        self,
        query_filter: IntegrationPackAtomAttachmentQueryConfigQueryFilter,
        **kwargs,
    ):
        """IntegrationPackAtomAttachmentQueryConfig

        :param query_filter: query_filter
        :type query_filter: IntegrationPackAtomAttachmentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, IntegrationPackAtomAttachmentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .api_usage_count_simple_expression import ApiUsageCountSimpleExpression
from .api_usage_count_grouping_expression import ApiUsageCountGroupingExpression


class ApiUsageCountExpressionGuard(OneOfBaseModel):
    class_list = {
        "ApiUsageCountSimpleExpression": ApiUsageCountSimpleExpression,
        "ApiUsageCountGroupingExpression": ApiUsageCountGroupingExpression,
    }


ApiUsageCountExpression = Union[
    ApiUsageCountSimpleExpression, ApiUsageCountGroupingExpression
]




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .custom_properties import CustomProperties


@JsonMap(
    {
        "component_override": "componentOverride",
        "custom_properties": "customProperties",
        "encrypted_value_set": "encryptedValueSet",
        "id_": "id",
        "use_default": "useDefault",
        "uses_encryption": "usesEncryption",
        "oauth2_authorization_url": "oauth2AuthorizationUrl",
    }
)
class FieldSummary(BaseModel):
    """FieldSummary

    :param component_override: component_override, defaults to None
    :type component_override: bool, optional
    :param custom_properties: custom_properties, defaults to None
    :type custom_properties: CustomProperties, optional
    :param encrypted_value_set: encrypted_value_set, defaults to None
    :type encrypted_value_set: bool, optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param use_default: use_default, defaults to None
    :type use_default: bool, optional
    :param uses_encryption: uses_encryption, defaults to None
    :type uses_encryption: bool, optional
    :param value: value, defaults to None
    :type value: str, optional
    :param oauth2_authorization_url: oauth2_authorization_url, defaults to None
    :type oauth2_authorization_url: str, optional
    """

    def __init__(
        self,
        component_override: bool = SENTINEL,
        custom_properties: CustomProperties = SENTINEL,
        encrypted_value_set: bool = SENTINEL,
        id_: str = SENTINEL,
        use_default: bool = SENTINEL,
        uses_encryption: bool = SENTINEL,
        value: str = SENTINEL,
        oauth2_authorization_url: str = SENTINEL,
        **kwargs,
    ):
        """FieldSummary

        :param component_override: component_override, defaults to None
        :type component_override: bool, optional
        :param custom_properties: custom_properties, defaults to None
        :type custom_properties: CustomProperties, optional
        :param encrypted_value_set: encrypted_value_set, defaults to None
        :type encrypted_value_set: bool, optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param use_default: use_default, defaults to None
        :type use_default: bool, optional
        :param uses_encryption: uses_encryption, defaults to None
        :type uses_encryption: bool, optional
        :param value: value, defaults to None
        :type value: str, optional
        :param oauth2_authorization_url: oauth2_authorization_url, defaults to None
        :type oauth2_authorization_url: str, optional
        """
        if component_override is not SENTINEL:
            self.component_override = component_override
        if custom_properties is not SENTINEL:
            self.custom_properties = self._define_object(
                custom_properties, CustomProperties
            )
        if encrypted_value_set is not SENTINEL:
            self.encrypted_value_set = encrypted_value_set
        if id_ is not SENTINEL:
            self.id_ = id_
        if use_default is not SENTINEL:
            self.use_default = use_default
        if uses_encryption is not SENTINEL:
            self.uses_encryption = uses_encryption
        if value is not SENTINEL:
            self.value = value
        if oauth2_authorization_url is not SENTINEL:
            self.oauth2_authorization_url = oauth2_authorization_url
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    Branch,
    BranchBulkRequest,
    BranchBulkResponse,
    BranchQueryConfig,
    BranchQueryResponse,
)


class BranchService(BaseService):

    @cast_models
    def create_branch(self, request_body: Branch = None) -> Union[Branch, str]:
        """- To create a branch, you need the branch ID for the branch from which you want to create a new branch. New branches return ready as false until the creating stage has cleared.
         - You can also create a branch from a packaged component. To do so, use the ID of the packaged component as the packageId.
         - To create a branch from a deployment, use the ID of the deployment for the packageId.

        :param request_body: The request body., defaults to None
        :type request_body: Branch, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Branch, str]
        """

        Validator(Branch).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Branch",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Branch._unmap(response)
        if content == "application/xml":
            return Branch._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_branch(self, id_: str) -> Union[Branch, str]:
        """When you have the branch ID, you can query for additional information about the branch. Send an HTTP GET where {accountId} is the ID of the authenticating account and {branchId} is the ID of the branch you want to query.

        :param id_: The ID of the branch.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Branch, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Branch/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Branch._unmap(response)
        if content == "application/xml":
            return Branch._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_branch(
        self, id_: str, request_body: Branch = None
    ) -> Union[Branch, str]:
        """To update a branch, you need the branch ID. Currently, you can only update the name of the branch.

        :param request_body: The request body., defaults to None
        :type request_body: Branch, optional
        :param id_: The ID of the branch.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Branch, str]
        """

        Validator(Branch).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Branch/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Branch._unmap(response)
        if content == "application/xml":
            return Branch._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_branch(self, id_: str) -> None:
        """Deletes a branch

        :param id_: The ID of the branch.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Branch/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_branch(
        self, request_body: BranchBulkRequest = None
    ) -> Union[BranchBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: BranchBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[BranchBulkResponse, str]
        """

        Validator(BranchBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Branch/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return BranchBulkResponse._unmap(response)
        if content == "application/xml":
            return BranchBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_branch(
        self, request_body: BranchQueryConfig = None
    ) -> Union[BranchQueryResponse, str]:
        """You must first retrieve the ID of your main branch, using the name of your current branch. If you haven't created any branches, your current branch will be `main`.

         When you query a branch, it might be in one of the following states:
         - `CREATING`: The branch is being created
         - `NORMAL`: The branch is ready to use
         - `DELETING`: The branch is being deleted.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: BranchQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[BranchQueryResponse, str]
        """

        Validator(BranchQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Branch/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return BranchQueryResponse._unmap(response)
        if content == "application/xml":
            return BranchQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_branch(self, request_body: str) -> Union[BranchQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[BranchQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Branch/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return BranchQueryResponse._unmap(response)
        if content == "application/xml":
            return BranchQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .process_property_value import ProcessPropertyValue


@JsonMap({"process_property_value": "ProcessPropertyValue", "id_": "id"})
class OverrideProcessProperty(BaseModel):
    """OverrideProcessProperty

    :param process_property_value: process_property_value, defaults to None
    :type process_property_value: List[ProcessPropertyValue], optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        process_property_value: List[ProcessPropertyValue] = SENTINEL,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        **kwargs,
    ):
        """OverrideProcessProperty

        :param process_property_value: process_property_value, defaults to None
        :type process_property_value: List[ProcessPropertyValue], optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param name: name, defaults to None
        :type name: str, optional
        """
        if process_property_value is not SENTINEL:
            self.process_property_value = self._define_list(
                process_property_value, ProcessPropertyValue
            )
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .as2_basic_auth_info import As2BasicAuthInfo
from .private_certificate import PrivateCertificate
from .public_certificate import PublicCertificate


class As2SendSettingsAuthenticationType(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "NONE"
    :vartype NONE: str
    :cvar BASIC: "BASIC"
    :vartype BASIC: str
    """

    NONE = "NONE"
    BASIC = "BASIC"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                As2SendSettingsAuthenticationType._member_map_.values(),
            )
        )


@JsonMap(
    {
        "auth_settings": "AuthSettings",
        "authentication_type": "authenticationType",
        "client_ssl_certificate": "clientSSLCertificate",
        "ssl_certificate": "sslCertificate",
        "use_default_settings": "useDefaultSettings",
        "verify_hostname": "verifyHostname",
    }
)
class As2SendSettings(BaseModel):
    """As2SendSettings

    :param auth_settings: auth_settings, defaults to None
    :type auth_settings: As2BasicAuthInfo, optional
    :param authentication_type: authentication_type, defaults to None
    :type authentication_type: As2SendSettingsAuthenticationType, optional
    :param client_ssl_certificate: client_ssl_certificate
    :type client_ssl_certificate: PrivateCertificate
    :param ssl_certificate: ssl_certificate
    :type ssl_certificate: PublicCertificate
    :param url: url
    :type url: str
    :param use_default_settings: use_default_settings, defaults to None
    :type use_default_settings: bool, optional
    :param verify_hostname: verify_hostname, defaults to None
    :type verify_hostname: bool, optional
    """

    def __init__(
        self,
        client_ssl_certificate: PrivateCertificate,
        ssl_certificate: PublicCertificate,
        url: str,
        auth_settings: As2BasicAuthInfo = SENTINEL,
        authentication_type: As2SendSettingsAuthenticationType = SENTINEL,
        use_default_settings: bool = SENTINEL,
        verify_hostname: bool = SENTINEL,
        **kwargs,
    ):
        """As2SendSettings

        :param auth_settings: auth_settings, defaults to None
        :type auth_settings: As2BasicAuthInfo, optional
        :param authentication_type: authentication_type, defaults to None
        :type authentication_type: As2SendSettingsAuthenticationType, optional
        :param client_ssl_certificate: client_ssl_certificate
        :type client_ssl_certificate: PrivateCertificate
        :param ssl_certificate: ssl_certificate
        :type ssl_certificate: PublicCertificate
        :param url: url
        :type url: str
        :param use_default_settings: use_default_settings, defaults to None
        :type use_default_settings: bool, optional
        :param verify_hostname: verify_hostname, defaults to None
        :type verify_hostname: bool, optional
        """
        if auth_settings is not SENTINEL:
            self.auth_settings = self._define_object(auth_settings, As2BasicAuthInfo)
        if authentication_type is not SENTINEL:
            self.authentication_type = self._enum_matching(
                authentication_type,
                As2SendSettingsAuthenticationType.list(),
                "authentication_type",
            )
        self.client_ssl_certificate = self._define_object(
            client_ssl_certificate, PrivateCertificate
        )
        self.ssl_certificate = self._define_object(ssl_certificate, PublicCertificate)
        self.url = url
        if use_default_settings is not SENTINEL:
            self.use_default_settings = use_default_settings
        if verify_hostname is not SENTINEL:
            self.verify_hostname = verify_hostname
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .as2_connector_record_simple_expression import As2ConnectorRecordSimpleExpression
from .as2_connector_record_grouping_expression import (
    As2ConnectorRecordGroupingExpression,
)


class As2ConnectorRecordExpressionGuard(OneOfBaseModel):
    class_list = {
        "As2ConnectorRecordSimpleExpression": As2ConnectorRecordSimpleExpression,
        "As2ConnectorRecordGroupingExpression": As2ConnectorRecordGroupingExpression,
    }


As2ConnectorRecordExpression = Union[
    As2ConnectorRecordSimpleExpression, As2ConnectorRecordGroupingExpression
]




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .disk_get_options import DiskGetOptions
from .disk_send_options import DiskSendOptions
from .shared_communication_channel import SharedCommunicationChannel


class DiskCommunicationOptionsCommunicationSetting(Enum):
    """An enumeration representing different categories.

    :cvar DEFAULT: "default"
    :vartype DEFAULT: str
    :cvar CUSTOM: "custom"
    :vartype CUSTOM: str
    :cvar COMPONENT: "component"
    :vartype COMPONENT: str
    """

    DEFAULT = "default"
    CUSTOM = "custom"
    COMPONENT = "component"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                DiskCommunicationOptionsCommunicationSetting._member_map_.values(),
            )
        )


@JsonMap(
    {
        "communication_setting": "CommunicationSetting",
        "disk_get_options": "DiskGetOptions",
        "disk_send_options": "DiskSendOptions",
        "shared_communication_channel": "SharedCommunicationChannel",
    }
)
class DiskCommunicationOptions(BaseModel):
    """DiskCommunicationOptions

    :param communication_setting: communication_setting, defaults to None
    :type communication_setting: DiskCommunicationOptionsCommunicationSetting, optional
    :param disk_get_options: disk_get_options, defaults to None
    :type disk_get_options: DiskGetOptions, optional
    :param disk_send_options: disk_send_options, defaults to None
    :type disk_send_options: DiskSendOptions, optional
    :param shared_communication_channel: shared_communication_channel, defaults to None
    :type shared_communication_channel: SharedCommunicationChannel, optional
    """

    def __init__(
        self,
        communication_setting: DiskCommunicationOptionsCommunicationSetting = SENTINEL,
        disk_get_options: DiskGetOptions = SENTINEL,
        disk_send_options: DiskSendOptions = SENTINEL,
        shared_communication_channel: SharedCommunicationChannel = SENTINEL,
        **kwargs,
    ):
        """DiskCommunicationOptions

        :param communication_setting: communication_setting, defaults to None
        :type communication_setting: DiskCommunicationOptionsCommunicationSetting, optional
        :param disk_get_options: disk_get_options, defaults to None
        :type disk_get_options: DiskGetOptions, optional
        :param disk_send_options: disk_send_options, defaults to None
        :type disk_send_options: DiskSendOptions, optional
        :param shared_communication_channel: shared_communication_channel, defaults to None
        :type shared_communication_channel: SharedCommunicationChannel, optional
        """
        if communication_setting is not SENTINEL:
            self.communication_setting = self._enum_matching(
                communication_setting,
                DiskCommunicationOptionsCommunicationSetting.list(),
                "communication_setting",
            )
        if disk_get_options is not SENTINEL:
            self.disk_get_options = self._define_object(
                disk_get_options, DiskGetOptions
            )
        if disk_send_options is not SENTINEL:
            self.disk_send_options = self._define_object(
                disk_send_options, DiskSendOptions
            )
        if shared_communication_channel is not SENTINEL:
            self.shared_communication_channel = self._define_object(
                shared_communication_channel, SharedCommunicationChannel
            )
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class EnvironmentClassification(Enum):
    """An enumeration representing different categories.

    :cvar PROD: "PROD"
    :vartype PROD: str
    :cvar TEST: "TEST"
    :vartype TEST: str
    """

    PROD = "PROD"
    TEST = "TEST"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, EnvironmentClassification._member_map_.values())
        )


@JsonMap(
    {
        "id_": "id",
        "parent_account": "parentAccount",
        "parent_environment": "parentEnvironment",
    }
)
class Environment(BaseModel):
    """Environment

    :param classification: \(Optional\) For accounts with Unlimited environment support, the type of environment.The choices are PROD \(Production\) and TEST. The environment classification determines the type of license used when deploying a process to the environment. The default classification is PROD.\<br /\>You can assign the value TEST only if the requesting account has Test Connection Licensing enabled.\<br /\>You can set the classification only when you add an environment. You cannot change the classification later.\<br /\>Environments added prior to the January 2014 release have their classification set to PROD.   \>**Note:** The classification field is invalid for requests from accounts with Basic environment support because all environments are production environments., defaults to None
    :type classification: EnvironmentClassification, optional
    :param id_: A unique ID assigned by the system to the environment., defaults to None
    :type id_: str, optional
    :param name: A user-defined name for the environment., defaults to None
    :type name: str, optional
    :param parent_account: parent_account, defaults to None
    :type parent_account: str, optional
    :param parent_environment: parent_environment, defaults to None
    :type parent_environment: str, optional
    """

    def __init__(
        self,
        classification: EnvironmentClassification = SENTINEL,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        parent_account: str = SENTINEL,
        parent_environment: str = SENTINEL,
        **kwargs
    ):
        """Environment

        :param classification: \(Optional\) For accounts with Unlimited environment support, the type of environment.The choices are PROD \(Production\) and TEST. The environment classification determines the type of license used when deploying a process to the environment. The default classification is PROD.\<br /\>You can assign the value TEST only if the requesting account has Test Connection Licensing enabled.\<br /\>You can set the classification only when you add an environment. You cannot change the classification later.\<br /\>Environments added prior to the January 2014 release have their classification set to PROD.   \>**Note:** The classification field is invalid for requests from accounts with Basic environment support because all environments are production environments., defaults to None
        :type classification: EnvironmentClassification, optional
        :param id_: A unique ID assigned by the system to the environment., defaults to None
        :type id_: str, optional
        :param name: A user-defined name for the environment., defaults to None
        :type name: str, optional
        :param parent_account: parent_account, defaults to None
        :type parent_account: str, optional
        :param parent_environment: parent_environment, defaults to None
        :type parent_environment: str, optional
        """
        if classification is not SENTINEL:
            self.classification = self._enum_matching(
                classification, EnvironmentClassification.list(), "classification"
            )
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        if parent_account is not SENTINEL:
            self.parent_account = parent_account
        if parent_environment is not SENTINEL:
            self.parent_environment = parent_environment
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .custom_fields import CustomFields


@JsonMap(
    {
        "account": "account",
        "action_type": "actionType",
        "application_reference": "applicationReference",
        "atom_id": "atomId",
        "connector_name": "connectorName",
        "connector_type": "connectorType",
        "custom_fields": "customFields",
        "date_": "date",
        "date_processed": "dateProcessed",
        "document_index": "documentIndex",
        "error_message": "errorMessage",
        "execution_id": "executionId",
        "file_generation_number": "fileGenerationNumber",
        "file_version_number": "fileVersionNumber",
        "id_": "id",
        "message_type": "messageType",
        "operation_name": "operationName",
        "receiver_code": "receiverCode",
        "receiver_name": "receiverName",
        "receiver_transmission_reference": "receiverTransmissionReference",
        "sender_code": "senderCode",
        "sender_name": "senderName",
        "sender_transmission_reference": "senderTransmissionReference",
        "size": "size",
        "successful": "successful",
        "time": "time",
        "transmission_priority_code": "transmissionPriorityCode",
        "validation_report": "validationReport",
        "validation_status": "validationStatus",
    }
)
class TradacomsConnectorRecord(BaseModel):
    """TradacomsConnectorRecord

    :param account: The ID of the account in which you ran this record.
    :type account: str
    :param action_type: The type of action with which this record corresponds — Send for an outbound interchange, Get for an inbound interchange using the Disk, FTP, or SFTP communication method, or Listen for an inbound interchange using the AS2 or HTTP communication method.
    :type action_type: str
    :param application_reference: The code that enables the receiving trading partner to identify whether the message contains data for a particular type of application., defaults to None
    :type application_reference: str, optional
    :param atom_id: The Runtime ID from which you process this record.
    :type atom_id: str
    :param connector_name: The value is Trading Partner for a Tradacoms trading partner Send operation, or Start for a Tradacoms trading partner Listen operation.
    :type connector_name: str
    :param connector_type: Tradacoms is the connector type for any record.
    :type connector_type: str
    :param custom_fields: custom_fields
    :type custom_fields: CustomFields
    :param date_: The date of the message delivery., defaults to None
    :type date_: str, optional
    :param date_processed: The processing date and time of this record. The format is *yyyy-MM-dd'T'HH:mm:ss'Z'*, such as 2022-08-10T15:32:00Z.
    :type date_processed: str
    :param document_index: The numerical index of this record in the run., defaults to None
    :type document_index: int, optional
    :param error_message: Any error message associated with this record. Omit this field for a successful interchange.
    :type error_message: str
    :param execution_id: The ID of the run.
    :type execution_id: str
    :param file_generation_number: The number generated by the sending trading partner to uniquely reference the transmitted file., defaults to None
    :type file_generation_number: str, optional
    :param file_version_number: The number generated to uniquely reference a copy of the transmitted file., defaults to None
    :type file_version_number: str, optional
    :param id_: The ID of this record.
    :type id_: str
    :param message_type: The code identifying the type of message., defaults to None
    :type message_type: str, optional
    :param operation_name: The name of the operation component from which you processed the record.
    :type operation_name: str
    :param receiver_code: The identifier code of the receiving trading partner., defaults to None
    :type receiver_code: str, optional
    :param receiver_name: The name of the receiving trading partner., defaults to None
    :type receiver_name: str, optional
    :param receiver_transmission_reference: The receiving trading partner’s reference for the message, if known to the sending trading partner., defaults to None
    :type receiver_transmission_reference: str, optional
    :param sender_code: The identifier code of the sending trading partner., defaults to None
    :type sender_code: str, optional
    :param sender_name: The name of the sending trading partner., defaults to None
    :type sender_name: str, optional
    :param sender_transmission_reference: The sending trading partner’s reference for the message., defaults to None
    :type sender_transmission_reference: str, optional
    :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
    :type size: int, optional
    :param successful: Whether the record is a success or error., defaults to None
    :type successful: bool, optional
    :param time: The time of the message delivery., defaults to None
    :type time: str, optional
    :param transmission_priority_code: The code for the message transmission priority., defaults to None
    :type transmission_priority_code: str, optional
    :param validation_report: The validation report, which contains descriptions of segment validation errors., defaults to None
    :type validation_report: str, optional
    :param validation_status: The validation status — either Success or Errors., defaults to None
    :type validation_status: str, optional
    """

    def __init__(
        self,
        account: str,
        action_type: str,
        atom_id: str,
        connector_name: str,
        connector_type: str,
        custom_fields: CustomFields,
        date_processed: str,
        error_message: str,
        execution_id: str,
        id_: str,
        operation_name: str,
        application_reference: str = SENTINEL,
        date_: str = SENTINEL,
        document_index: int = SENTINEL,
        file_generation_number: str = SENTINEL,
        file_version_number: str = SENTINEL,
        message_type: str = SENTINEL,
        receiver_code: str = SENTINEL,
        receiver_name: str = SENTINEL,
        receiver_transmission_reference: str = SENTINEL,
        sender_code: str = SENTINEL,
        sender_name: str = SENTINEL,
        sender_transmission_reference: str = SENTINEL,
        size: int = SENTINEL,
        successful: bool = SENTINEL,
        time: str = SENTINEL,
        transmission_priority_code: str = SENTINEL,
        validation_report: str = SENTINEL,
        validation_status: str = SENTINEL,
        **kwargs,
    ):
        """TradacomsConnectorRecord

        :param account: The ID of the account in which you ran this record.
        :type account: str
        :param action_type: The type of action with which this record corresponds — Send for an outbound interchange, Get for an inbound interchange using the Disk, FTP, or SFTP communication method, or Listen for an inbound interchange using the AS2 or HTTP communication method.
        :type action_type: str
        :param application_reference: The code that enables the receiving trading partner to identify whether the message contains data for a particular type of application., defaults to None
        :type application_reference: str, optional
        :param atom_id: The Runtime ID from which you process this record.
        :type atom_id: str
        :param connector_name: The value is Trading Partner for a Tradacoms trading partner Send operation, or Start for a Tradacoms trading partner Listen operation.
        :type connector_name: str
        :param connector_type: Tradacoms is the connector type for any record.
        :type connector_type: str
        :param custom_fields: custom_fields
        :type custom_fields: CustomFields
        :param date_: The date of the message delivery., defaults to None
        :type date_: str, optional
        :param date_processed: The processing date and time of this record. The format is *yyyy-MM-dd'T'HH:mm:ss'Z'*, such as 2022-08-10T15:32:00Z.
        :type date_processed: str
        :param document_index: The numerical index of this record in the run., defaults to None
        :type document_index: int, optional
        :param error_message: Any error message associated with this record. Omit this field for a successful interchange.
        :type error_message: str
        :param execution_id: The ID of the run.
        :type execution_id: str
        :param file_generation_number: The number generated by the sending trading partner to uniquely reference the transmitted file., defaults to None
        :type file_generation_number: str, optional
        :param file_version_number: The number generated to uniquely reference a copy of the transmitted file., defaults to None
        :type file_version_number: str, optional
        :param id_: The ID of this record.
        :type id_: str
        :param message_type: The code identifying the type of message., defaults to None
        :type message_type: str, optional
        :param operation_name: The name of the operation component from which you processed the record.
        :type operation_name: str
        :param receiver_code: The identifier code of the receiving trading partner., defaults to None
        :type receiver_code: str, optional
        :param receiver_name: The name of the receiving trading partner., defaults to None
        :type receiver_name: str, optional
        :param receiver_transmission_reference: The receiving trading partner’s reference for the message, if known to the sending trading partner., defaults to None
        :type receiver_transmission_reference: str, optional
        :param sender_code: The identifier code of the sending trading partner., defaults to None
        :type sender_code: str, optional
        :param sender_name: The name of the sending trading partner., defaults to None
        :type sender_name: str, optional
        :param sender_transmission_reference: The sending trading partner’s reference for the message., defaults to None
        :type sender_transmission_reference: str, optional
        :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
        :type size: int, optional
        :param successful: Whether the record is a success or error., defaults to None
        :type successful: bool, optional
        :param time: The time of the message delivery., defaults to None
        :type time: str, optional
        :param transmission_priority_code: The code for the message transmission priority., defaults to None
        :type transmission_priority_code: str, optional
        :param validation_report: The validation report, which contains descriptions of segment validation errors., defaults to None
        :type validation_report: str, optional
        :param validation_status: The validation status — either Success or Errors., defaults to None
        :type validation_status: str, optional
        """
        self.account = account
        self.action_type = action_type
        if application_reference is not SENTINEL:
            self.application_reference = application_reference
        self.atom_id = atom_id
        self.connector_name = connector_name
        self.connector_type = connector_type
        self.custom_fields = self._define_object(custom_fields, CustomFields)
        if date_ is not SENTINEL:
            self.date_ = date_
        self.date_processed = date_processed
        if document_index is not SENTINEL:
            self.document_index = document_index
        self.error_message = error_message
        self.execution_id = execution_id
        if file_generation_number is not SENTINEL:
            self.file_generation_number = file_generation_number
        if file_version_number is not SENTINEL:
            self.file_version_number = file_version_number
        self.id_ = id_
        if message_type is not SENTINEL:
            self.message_type = message_type
        self.operation_name = operation_name
        if receiver_code is not SENTINEL:
            self.receiver_code = receiver_code
        if receiver_name is not SENTINEL:
            self.receiver_name = receiver_name
        if receiver_transmission_reference is not SENTINEL:
            self.receiver_transmission_reference = receiver_transmission_reference
        if sender_code is not SENTINEL:
            self.sender_code = sender_code
        if sender_name is not SENTINEL:
            self.sender_name = sender_name
        if sender_transmission_reference is not SENTINEL:
            self.sender_transmission_reference = sender_transmission_reference
        if size is not SENTINEL:
            self.size = size
        if successful is not SENTINEL:
            self.successful = successful
        if time is not SENTINEL:
            self.time = time
        if transmission_priority_code is not SENTINEL:
            self.transmission_priority_code = transmission_priority_code
        if validation_report is not SENTINEL:
            self.validation_report = validation_report
        if validation_status is not SENTINEL:
            self.validation_status = validation_status
        self._kwargs = kwargs




from typing import Awaitable
from .utils.to_async import to_async
from ..worker import WorkerService


class WorkerServiceAsync(WorkerService):
    """
    Async Wrapper for WorkerServiceAsync
    """

    def create_worker(self) -> Awaitable[None]:
        return to_async(super().create_worker)()




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .license import License


@JsonMap(
    {
        "enterprise_test": "enterpriseTest",
        "small_business": "smallBusiness",
        "small_business_test": "smallBusinessTest",
        "standard_test": "standardTest",
        "trading_partner": "tradingPartner",
        "trading_partner_test": "tradingPartnerTest",
    }
)
class Licensing(BaseModel):
    """Indicates the number of connections used and purchased in each of the connector type and production/test classifications. The classifications include standard, smallBusiness, enterprise, and tradingPartner.

    :param enterprise: enterprise, defaults to None
    :type enterprise: License, optional
    :param enterprise_test: enterprise_test, defaults to None
    :type enterprise_test: License, optional
    :param small_business: small_business, defaults to None
    :type small_business: License, optional
    :param small_business_test: small_business_test, defaults to None
    :type small_business_test: License, optional
    :param standard: standard, defaults to None
    :type standard: License, optional
    :param standard_test: standard_test, defaults to None
    :type standard_test: License, optional
    :param trading_partner: trading_partner, defaults to None
    :type trading_partner: License, optional
    :param trading_partner_test: trading_partner_test, defaults to None
    :type trading_partner_test: License, optional
    """

    def __init__(
        self,
        enterprise: License = SENTINEL,
        enterprise_test: License = SENTINEL,
        small_business: License = SENTINEL,
        small_business_test: License = SENTINEL,
        standard: License = SENTINEL,
        standard_test: License = SENTINEL,
        trading_partner: License = SENTINEL,
        trading_partner_test: License = SENTINEL,
        **kwargs,
    ):
        """Indicates the number of connections used and purchased in each of the connector type and production/test classifications. The classifications include standard, smallBusiness, enterprise, and tradingPartner.

        :param enterprise: enterprise, defaults to None
        :type enterprise: License, optional
        :param enterprise_test: enterprise_test, defaults to None
        :type enterprise_test: License, optional
        :param small_business: small_business, defaults to None
        :type small_business: License, optional
        :param small_business_test: small_business_test, defaults to None
        :type small_business_test: License, optional
        :param standard: standard, defaults to None
        :type standard: License, optional
        :param standard_test: standard_test, defaults to None
        :type standard_test: License, optional
        :param trading_partner: trading_partner, defaults to None
        :type trading_partner: License, optional
        :param trading_partner_test: trading_partner_test, defaults to None
        :type trading_partner_test: License, optional
        """
        if enterprise is not SENTINEL:
            self.enterprise = self._define_object(enterprise, License)
        if enterprise_test is not SENTINEL:
            self.enterprise_test = self._define_object(enterprise_test, License)
        if small_business is not SENTINEL:
            self.small_business = self._define_object(small_business, License)
        if small_business_test is not SENTINEL:
            self.small_business_test = self._define_object(small_business_test, License)
        if standard is not SENTINEL:
            self.standard = self._define_object(standard, License)
        if standard_test is not SENTINEL:
            self.standard_test = self._define_object(standard_test, License)
        if trading_partner is not SENTINEL:
            self.trading_partner = self._define_object(trading_partner, License)
        if trading_partner_test is not SENTINEL:
            self.trading_partner_test = self._define_object(
                trading_partner_test, License
            )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .httpssl_options import HttpsslOptions


@JsonMap({"ssl_options": "sslOptions"})
class HttpEndpoint(BaseModel):
    """HttpEndpoint

    :param ssl_options: ssl_options
    :type ssl_options: HttpsslOptions
    :param url: url, defaults to None
    :type url: str, optional
    """

    def __init__(self, ssl_options: HttpsslOptions, url: str = SENTINEL, **kwargs):
        """HttpEndpoint

        :param ssl_options: ssl_options
        :type ssl_options: HttpsslOptions
        :param url: url, defaults to None
        :type url: str, optional
        """
        self.ssl_options = self._define_object(ssl_options, HttpsslOptions)
        if url is not SENTINEL:
            self.url = url
        self._kwargs = kwargs
